lilypond_version = "2.19.24"

contexts = {
    "ChoirStaff": {
        "accepts": set([
            "ChoirStaff",
            "ChordNames",
            "DrumStaff",
            "FiguredBass",
            "GrandStaff",
            "Lyrics",
            "PianoStaff",
            "RhythmicStaff",
            "Staff",
            "StaffGroup",
            ]),
        "aliases": set([]),
        "consists": set([
            "Instrument_name_engraver",
            "System_start_delimiter_engraver",
            "Vertical_align_engraver",
            ]),
        "default_child": "Staff",
        },
    "ChordNames": {
        "accepts": set([]),
        "aliases": set([
            "Staff",
            ]),
        "consists": set([
            "Axis_group_engraver",
            "Chord_name_engraver",
            "Output_property_engraver",
            "Separating_line_group_engraver",
            ]),
        },
    "CueVoice": {
        "accepts": set([]),
        "aliases": set([
            "Voice",
            ]),
        "consists": set([
            "Arpeggio_engraver",
            "Auto_beam_engraver",
            "Beam_engraver",
            "Bend_engraver",
            "Breathing_sign_engraver",
            "Chord_tremolo_engraver",
            "Cluster_spanner_engraver",
            "Dots_engraver",
            "Double_percent_repeat_engraver",
            "Dynamic_align_engraver",
            "Dynamic_engraver",
            "Fingering_engraver",
            "Font_size_engraver",
            "Forbid_line_break_engraver",
            "Glissando_engraver",
            "Grace_auto_beam_engraver",
            "Grace_beam_engraver",
            "Grace_engraver",
            "Grob_pq_engraver",
            "Instrument_switch_engraver",
            "Laissez_vibrer_engraver",
            "Ligature_bracket_engraver",
            "Multi_measure_rest_engraver",
            "New_fingering_engraver",
            "Note_head_line_engraver",
            "Note_heads_engraver",
            "Note_spacing_engraver",
            "Output_property_engraver",
            "Part_combine_engraver",
            "Percent_repeat_engraver",
            "Phrasing_slur_engraver",
            "Pitched_trill_engraver",
            "Repeat_tie_engraver",
            "Rest_engraver",
            "Rhythmic_column_engraver",
            "Script_column_engraver",
            "Script_engraver",
            "Slash_repeat_engraver",
            "Slur_engraver",
            "Spanner_break_forbid_engraver",
            "Stem_engraver",
            "Text_engraver",
            "Text_spanner_engraver",
            "Tie_engraver",
            "Trill_spanner_engraver",
            "Tuplet_engraver",
            ]),
        },
    "Devnull": {
        "accepts": set([]),
        "aliases": set([
            "Staff",
            "Voice",
            ]),
        "consists": set([]),
        },
    "DrumStaff": {
        "accepts": set([
            "CueVoice",
            "DrumVoice",
            "NullVoice",
            ]),
        "aliases": set([
            "Staff",
            ]),
        "consists": set([
            "Axis_group_engraver",
            "Bar_engraver",
            "Clef_engraver",
            "Collision_engraver",
            "Cue_clef_engraver",
            "Dot_column_engraver",
            "Figured_bass_engraver",
            "Figured_bass_position_engraver",
            "Fingering_column_engraver",
            "Font_size_engraver",
            "Grob_pq_engraver",
            "Instrument_name_engraver",
            "Ledger_line_engraver",
            "Output_property_engraver",
            "Piano_pedal_align_engraver",
            "Pure_from_neighbor_engraver",
            "Rest_collision_engraver",
            "Script_row_engraver",
            "Separating_line_group_engraver",
            "Staff_collecting_engraver",
            "Staff_symbol_engraver",
            "Time_signature_engraver",
            ]),
        "default_child": "DrumVoice",
        },
    "DrumVoice": {
        "accepts": set([]),
        "aliases": set([
            "Voice",
            ]),
        "consists": set([
            "Auto_beam_engraver",
            "Beam_engraver",
            "Bend_engraver",
            "Breathing_sign_engraver",
            "Chord_tremolo_engraver",
            "Dots_engraver",
            "Double_percent_repeat_engraver",
            "Drum_notes_engraver",
            "Dynamic_align_engraver",
            "Dynamic_engraver",
            "Font_size_engraver",
            "Forbid_line_break_engraver",
            "Grace_auto_beam_engraver",
            "Grace_beam_engraver",
            "Grace_engraver",
            "Grob_pq_engraver",
            "Grob_pq_engraver",
            "Instrument_switch_engraver",
            "Laissez_vibrer_engraver",
            "Multi_measure_rest_engraver",
            "Note_spacing_engraver",
            "Output_property_engraver",
            "Part_combine_engraver",
            "Percent_repeat_engraver",
            "Phrasing_slur_engraver",
            "Pitched_trill_engraver",
            "Repeat_tie_engraver",
            "Rest_engraver",
            "Rhythmic_column_engraver",
            "Script_column_engraver",
            "Script_engraver",
            "Slash_repeat_engraver",
            "Slur_engraver",
            "Spanner_break_forbid_engraver",
            "Stem_engraver",
            "Text_engraver",
            "Text_spanner_engraver",
            "Tie_engraver",
            "Trill_spanner_engraver",
            "Tuplet_engraver",
            ]),
        },
    "Dynamics": {
        "accepts": set([]),
        "aliases": set([
            "Voice",
            ]),
        "consists": set([
            "Axis_group_engraver",
            "Bar_engraver",
            "Dynamic_align_engraver",
            "Dynamic_engraver",
            "Font_size_engraver",
            "Output_property_engraver",
            "Piano_pedal_engraver",
            "Script_engraver",
            "Text_engraver",
            "Text_spanner_engraver",
            ]),
        },
    "FiguredBass": {
        "accepts": set([]),
        "aliases": set([]),
        "consists": set([
            "Axis_group_engraver",
            "Figured_bass_engraver",
            "Separating_line_group_engraver",
            ]),
        },
    "FretBoards": {
        "accepts": set([]),
        "aliases": set([
            "Staff",
            ]),
        "consists": set([
            "Axis_group_engraver",
            "Font_size_engraver",
            "Fretboard_engraver",
            "Instrument_name_engraver",
            "Output_property_engraver",
            "Separating_line_group_engraver",
            ]),
        },
    "Global": {
        "accepts": set([
            "Score",
            ]),
        "aliases": set([]),
        "consists": set([]),
        "default_child": "Score",
        },
    "GrandStaff": {
        "accepts": set([
            "ChordNames",
            "DrumStaff",
            "Dynamics",
            "FiguredBass",
            "Lyrics",
            "RhythmicStaff",
            "Staff",
            "TabStaff",
            ]),
        "aliases": set([]),
        "consists": set([
            "Instrument_name_engraver",
            "Span_arpeggio_engraver",
            "Span_bar_engraver",
            "Span_bar_stub_engraver",
            "System_start_delimiter_engraver",
            "Vertical_align_engraver",
            ]),
        "default_child": "Staff",
        },
    "GregorianTranscriptionStaff": {
        "accepts": set([
            "CueVoice",
            "GregorianTranscriptionVoice",
            "NullVoice",
            ]),
        "aliases": set([
            "Staff",
            ]),
        "consists": set([
            "Accidental_engraver",
            "Axis_group_engraver",
            "Bar_engraver",
            "Clef_engraver",
            "Collision_engraver",
            "Cue_clef_engraver",
            "Dot_column_engraver",
            "Figured_bass_engraver",
            "Figured_bass_position_engraver",
            "Fingering_column_engraver",
            "Font_size_engraver",
            "Grob_pq_engraver",
            "Instrument_name_engraver",
            "Key_engraver",
            "Ledger_line_engraver",
            "Ottava_spanner_engraver",
            "Output_property_engraver",
            "Piano_pedal_align_engraver",
            "Piano_pedal_engraver",
            "Pure_from_neighbor_engraver",
            "Rest_collision_engraver",
            "Script_row_engraver",
            "Separating_line_group_engraver",
            "Staff_collecting_engraver",
            "Staff_symbol_engraver",
            "Time_signature_engraver",
            ]),
        "default_child": "GregorianTranscriptionVoice",
        },
    "GregorianTranscriptionVoice": {
        "accepts": set([]),
        "aliases": set([
            "Voice",
            ]),
        "consists": set([
            "Arpeggio_engraver",
            "Auto_beam_engraver",
            "Beam_engraver",
            "Bend_engraver",
            "Breathing_sign_engraver",
            "Chord_tremolo_engraver",
            "Cluster_spanner_engraver",
            "Dots_engraver",
            "Double_percent_repeat_engraver",
            "Dynamic_align_engraver",
            "Dynamic_engraver",
            "Episema_engraver",
            "Fingering_engraver",
            "Font_size_engraver",
            "Forbid_line_break_engraver",
            "Glissando_engraver",
            "Grace_auto_beam_engraver",
            "Grace_beam_engraver",
            "Grace_engraver",
            "Grob_pq_engraver",
            "Instrument_switch_engraver",
            "Laissez_vibrer_engraver",
            "Ligature_bracket_engraver",
            "Multi_measure_rest_engraver",
            "New_fingering_engraver",
            "Note_head_line_engraver",
            "Note_heads_engraver",
            "Note_spacing_engraver",
            "Output_property_engraver",
            "Part_combine_engraver",
            "Percent_repeat_engraver",
            "Phrasing_slur_engraver",
            "Pitched_trill_engraver",
            "Repeat_tie_engraver",
            "Rest_engraver",
            "Rhythmic_column_engraver",
            "Script_column_engraver",
            "Script_engraver",
            "Slash_repeat_engraver",
            "Slur_engraver",
            "Spanner_break_forbid_engraver",
            "Stem_engraver",
            "Text_engraver",
            "Text_spanner_engraver",
            "Tie_engraver",
            "Trill_spanner_engraver",
            "Tuplet_engraver",
            ]),
        },
    "KievanStaff": {
        "accepts": set([
            "CueVoice",
            "KievanVoice",
            "NullVoice",
            ]),
        "aliases": set([
            "Staff",
            ]),
        "consists": set([
            "Accidental_engraver",
            "Axis_group_engraver",
            "Bar_engraver",
            "Clef_engraver",
            "Collision_engraver",
            "Cue_clef_engraver",
            "Dot_column_engraver",
            "Figured_bass_engraver",
            "Figured_bass_position_engraver",
            "Fingering_column_engraver",
            "Font_size_engraver",
            "Grob_pq_engraver",
            "Instrument_name_engraver",
            "Key_engraver",
            "Ledger_line_engraver",
            "Ottava_spanner_engraver",
            "Output_property_engraver",
            "Piano_pedal_align_engraver",
            "Piano_pedal_engraver",
            "Pure_from_neighbor_engraver",
            "Rest_collision_engraver",
            "Script_row_engraver",
            "Separating_line_group_engraver",
            "Staff_collecting_engraver",
            "Staff_symbol_engraver",
            ]),
        "default_child": "KievanVoice",
        },
    "KievanVoice": {
        "accepts": set([]),
        "aliases": set([
            "Voice",
            ]),
        "consists": set([
            "Arpeggio_engraver",
            "Auto_beam_engraver",
            "Beam_engraver",
            "Bend_engraver",
            "Breathing_sign_engraver",
            "Chord_tremolo_engraver",
            "Cluster_spanner_engraver",
            "Dots_engraver",
            "Double_percent_repeat_engraver",
            "Dynamic_align_engraver",
            "Dynamic_engraver",
            "Fingering_engraver",
            "Font_size_engraver",
            "Forbid_line_break_engraver",
            "Glissando_engraver",
            "Grace_auto_beam_engraver",
            "Grace_beam_engraver",
            "Grace_engraver",
            "Grob_pq_engraver",
            "Instrument_switch_engraver",
            "Kievan_ligature_engraver",
            "Laissez_vibrer_engraver",
            "Multi_measure_rest_engraver",
            "New_fingering_engraver",
            "Note_head_line_engraver",
            "Note_heads_engraver",
            "Note_spacing_engraver",
            "Output_property_engraver",
            "Part_combine_engraver",
            "Percent_repeat_engraver",
            "Phrasing_slur_engraver",
            "Pitched_trill_engraver",
            "Repeat_tie_engraver",
            "Rest_engraver",
            "Rhythmic_column_engraver",
            "Script_column_engraver",
            "Script_engraver",
            "Slash_repeat_engraver",
            "Slur_engraver",
            "Spanner_break_forbid_engraver",
            "Stem_engraver",
            "Text_engraver",
            "Text_spanner_engraver",
            "Tie_engraver",
            "Trill_spanner_engraver",
            "Tuplet_engraver",
            ]),
        },
    "Lyrics": {
        "accepts": set([]),
        "aliases": set([]),
        "consists": set([
            "Axis_group_engraver",
            "Extender_engraver",
            "Font_size_engraver",
            "Hyphen_engraver",
            "Instrument_name_engraver",
            "Lyric_engraver",
            "Pure_from_neighbor_engraver",
            "Stanza_number_engraver",
            ]),
        },
    "MensuralStaff": {
        "accepts": set([
            "CueVoice",
            "MensuralVoice",
            "NullVoice",
            ]),
        "aliases": set([
            "Staff",
            ]),
        "consists": set([
            "Accidental_engraver",
            "Axis_group_engraver",
            "Bar_engraver",
            "Clef_engraver",
            "Collision_engraver",
            "Cue_clef_engraver",
            "Custos_engraver",
            "Dot_column_engraver",
            "Figured_bass_engraver",
            "Figured_bass_position_engraver",
            "Fingering_column_engraver",
            "Font_size_engraver",
            "Grob_pq_engraver",
            "Instrument_name_engraver",
            "Key_engraver",
            "Ledger_line_engraver",
            "Ottava_spanner_engraver",
            "Output_property_engraver",
            "Piano_pedal_align_engraver",
            "Piano_pedal_engraver",
            "Pure_from_neighbor_engraver",
            "Rest_collision_engraver",
            "Script_row_engraver",
            "Separating_line_group_engraver",
            "Staff_collecting_engraver",
            "Staff_symbol_engraver",
            "Time_signature_engraver",
            ]),
        "default_child": "MensuralVoice",
        },
    "MensuralVoice": {
        "accepts": set([]),
        "aliases": set([
            "Voice",
            ]),
        "consists": set([
            "Arpeggio_engraver",
            "Auto_beam_engraver",
            "Beam_engraver",
            "Bend_engraver",
            "Breathing_sign_engraver",
            "Chord_tremolo_engraver",
            "Cluster_spanner_engraver",
            "Dots_engraver",
            "Double_percent_repeat_engraver",
            "Dynamic_align_engraver",
            "Dynamic_engraver",
            "Fingering_engraver",
            "Font_size_engraver",
            "Forbid_line_break_engraver",
            "Glissando_engraver",
            "Grace_auto_beam_engraver",
            "Grace_beam_engraver",
            "Grace_engraver",
            "Grob_pq_engraver",
            "Instrument_switch_engraver",
            "Laissez_vibrer_engraver",
            "Mensural_ligature_engraver",
            "Multi_measure_rest_engraver",
            "New_fingering_engraver",
            "Note_head_line_engraver",
            "Note_heads_engraver",
            "Note_spacing_engraver",
            "Output_property_engraver",
            "Part_combine_engraver",
            "Percent_repeat_engraver",
            "Phrasing_slur_engraver",
            "Pitched_trill_engraver",
            "Repeat_tie_engraver",
            "Rest_engraver",
            "Rhythmic_column_engraver",
            "Script_column_engraver",
            "Script_engraver",
            "Slash_repeat_engraver",
            "Spanner_break_forbid_engraver",
            "Stem_engraver",
            "Text_engraver",
            "Text_spanner_engraver",
            "Tie_engraver",
            "Trill_spanner_engraver",
            "Tuplet_engraver",
            ]),
        },
    "NoteNames": {
        "accepts": set([]),
        "aliases": set([]),
        "consists": set([
            "Axis_group_engraver",
            "Note_name_engraver",
            "Separating_line_group_engraver",
            "Tie_engraver",
            ]),
        },
    "NullVoice": {
        "accepts": set([]),
        "aliases": set([
            "Staff",
            "Voice",
            ]),
        "consists": set([
            "Beam_engraver",
            "Grob_pq_engraver",
            "Note_heads_engraver",
            "Pitch_squash_engraver",
            "Slur_engraver",
            "Tie_engraver",
            ]),
        },
    "PetrucciStaff": {
        "accepts": set([
            "CueVoice",
            "NullVoice",
            "PetrucciVoice",
            ]),
        "aliases": set([
            "Staff",
            ]),
        "consists": set([
            "Accidental_engraver",
            "Axis_group_engraver",
            "Bar_engraver",
            "Clef_engraver",
            "Collision_engraver",
            "Cue_clef_engraver",
            "Custos_engraver",
            "Dot_column_engraver",
            "Figured_bass_engraver",
            "Figured_bass_position_engraver",
            "Fingering_column_engraver",
            "Font_size_engraver",
            "Grob_pq_engraver",
            "Instrument_name_engraver",
            "Key_engraver",
            "Ledger_line_engraver",
            "Ottava_spanner_engraver",
            "Output_property_engraver",
            "Piano_pedal_align_engraver",
            "Piano_pedal_engraver",
            "Pure_from_neighbor_engraver",
            "Rest_collision_engraver",
            "Script_row_engraver",
            "Separating_line_group_engraver",
            "Staff_collecting_engraver",
            "Staff_symbol_engraver",
            "Time_signature_engraver",
            ]),
        "default_child": "PetrucciVoice",
        },
    "PetrucciVoice": {
        "accepts": set([]),
        "aliases": set([
            "Voice",
            ]),
        "consists": set([
            "Arpeggio_engraver",
            "Auto_beam_engraver",
            "Beam_engraver",
            "Bend_engraver",
            "Breathing_sign_engraver",
            "Chord_tremolo_engraver",
            "Cluster_spanner_engraver",
            "Dots_engraver",
            "Double_percent_repeat_engraver",
            "Dynamic_align_engraver",
            "Dynamic_engraver",
            "Fingering_engraver",
            "Font_size_engraver",
            "Forbid_line_break_engraver",
            "Glissando_engraver",
            "Grace_auto_beam_engraver",
            "Grace_beam_engraver",
            "Grace_engraver",
            "Grob_pq_engraver",
            "Instrument_switch_engraver",
            "Laissez_vibrer_engraver",
            "Mensural_ligature_engraver",
            "Multi_measure_rest_engraver",
            "New_fingering_engraver",
            "Note_head_line_engraver",
            "Note_heads_engraver",
            "Note_spacing_engraver",
            "Output_property_engraver",
            "Part_combine_engraver",
            "Percent_repeat_engraver",
            "Phrasing_slur_engraver",
            "Pitched_trill_engraver",
            "Repeat_tie_engraver",
            "Rest_engraver",
            "Rhythmic_column_engraver",
            "Script_column_engraver",
            "Script_engraver",
            "Slash_repeat_engraver",
            "Slur_engraver",
            "Spanner_break_forbid_engraver",
            "Stem_engraver",
            "Text_engraver",
            "Text_spanner_engraver",
            "Tie_engraver",
            "Trill_spanner_engraver",
            "Tuplet_engraver",
            ]),
        },
    "PianoStaff": {
        "accepts": set([
            "ChordNames",
            "DrumStaff",
            "Dynamics",
            "FiguredBass",
            "Lyrics",
            "RhythmicStaff",
            "Staff",
            "TabStaff",
            ]),
        "aliases": set([
            "GrandStaff",
            ]),
        "consists": set([
            "Instrument_name_engraver",
            "Keep_alive_together_engraver",
            "Span_arpeggio_engraver",
            "Span_bar_engraver",
            "Span_bar_stub_engraver",
            "System_start_delimiter_engraver",
            "Vertical_align_engraver",
            "Vertical_align_engraver",
            ]),
        "default_child": "Staff",
        },
    "RhythmicStaff": {
        "accepts": set([
            "CueVoice",
            "NullVoice",
            "Voice",
            ]),
        "aliases": set([
            "Staff",
            ]),
        "consists": set([
            "Axis_group_engraver",
            "Bar_engraver",
            "Dot_column_engraver",
            "Font_size_engraver",
            "Instrument_name_engraver",
            "Ledger_line_engraver",
            "Output_property_engraver",
            "Pitch_squash_engraver",
            "Separating_line_group_engraver",
            "Staff_symbol_engraver",
            "Time_signature_engraver",
            ]),
        "default_child": "Voice",
        },
    "Score": {
        "accepts": set([
            "ChoirStaff",
            "ChordNames",
            "Devnull",
            "DrumStaff",
            "FiguredBass",
            "FretBoards",
            "GrandStaff",
            "GregorianTranscriptionStaff",
            "KievanStaff",
            "Lyrics",
            "MensuralStaff",
            "NoteNames",
            "PetrucciStaff",
            "PianoStaff",
            "RhythmicStaff",
            "Staff",
            "StaffGroup",
            "TabStaff",
            "VaticanaStaff",
            ]),
        "aliases": set([
            "Timing",
            ]),
        "consists": set([
            "Bar_number_engraver",
            "Beam_collision_engraver",
            "Break_align_engraver",
            "Concurrent_hairpin_engraver",
            "Default_bar_line_engraver",
            "Footnote_engraver",
            "Grace_spacing_engraver",
            "Mark_engraver",
            "Metronome_mark_engraver",
            "Output_property_engraver",
            "Paper_column_engraver",
            "Parenthesis_engraver",
            "Repeat_acknowledge_engraver",
            "Spacing_engraver",
            "Staff_collecting_engraver",
            "Stanza_number_align_engraver",
            "System_start_delimiter_engraver",
            "Timing_translator",
            "Tweak_engraver",
            "Vertical_align_engraver",
            "Volta_engraver",
            ]),
        "default_child": "Staff",
        },
    "Staff": {
        "accepts": set([
            "CueVoice",
            "NullVoice",
            "Voice",
            ]),
        "aliases": set([]),
        "consists": set([
            "Accidental_engraver",
            "Axis_group_engraver",
            "Bar_engraver",
            "Clef_engraver",
            "Collision_engraver",
            "Cue_clef_engraver",
            "Dot_column_engraver",
            "Figured_bass_engraver",
            "Figured_bass_position_engraver",
            "Fingering_column_engraver",
            "Font_size_engraver",
            "Grob_pq_engraver",
            "Instrument_name_engraver",
            "Key_engraver",
            "Ledger_line_engraver",
            "Ottava_spanner_engraver",
            "Output_property_engraver",
            "Piano_pedal_align_engraver",
            "Piano_pedal_engraver",
            "Pure_from_neighbor_engraver",
            "Rest_collision_engraver",
            "Script_row_engraver",
            "Separating_line_group_engraver",
            "Staff_collecting_engraver",
            "Staff_symbol_engraver",
            "Time_signature_engraver",
            ]),
        "default_child": "Voice",
        },
    "StaffGroup": {
        "accepts": set([
            "ChoirStaff",
            "ChordNames",
            "DrumStaff",
            "FiguredBass",
            "FretBoards",
            "GrandStaff",
            "Lyrics",
            "PianoStaff",
            "RhythmicStaff",
            "Staff",
            "StaffGroup",
            "TabStaff",
            ]),
        "aliases": set([]),
        "consists": set([
            "Instrument_name_engraver",
            "Output_property_engraver",
            "Span_arpeggio_engraver",
            "Span_bar_engraver",
            "Span_bar_stub_engraver",
            "System_start_delimiter_engraver",
            "Vertical_align_engraver",
            ]),
        "default_child": "Staff",
        },
    "TabStaff": {
        "accepts": set([
            "CueVoice",
            "NullVoice",
            "TabVoice",
            ]),
        "aliases": set([
            "Staff",
            ]),
        "consists": set([
            "Axis_group_engraver",
            "Bar_engraver",
            "Clef_engraver",
            "Collision_engraver",
            "Cue_clef_engraver",
            "Dot_column_engraver",
            "Figured_bass_engraver",
            "Figured_bass_position_engraver",
            "Fingering_column_engraver",
            "Font_size_engraver",
            "Grob_pq_engraver",
            "Instrument_name_engraver",
            "Ledger_line_engraver",
            "Output_property_engraver",
            "Piano_pedal_align_engraver",
            "Piano_pedal_engraver",
            "Pure_from_neighbor_engraver",
            "Rest_collision_engraver",
            "Script_row_engraver",
            "Separating_line_group_engraver",
            "Staff_collecting_engraver",
            "Staff_symbol_engraver",
            "Tab_staff_symbol_engraver",
            "Time_signature_engraver",
            ]),
        "default_child": "TabVoice",
        },
    "TabVoice": {
        "accepts": set([]),
        "aliases": set([
            "Voice",
            ]),
        "consists": set([
            "Arpeggio_engraver",
            "Auto_beam_engraver",
            "Beam_engraver",
            "Bend_engraver",
            "Breathing_sign_engraver",
            "Chord_tremolo_engraver",
            "Cluster_spanner_engraver",
            "Dots_engraver",
            "Double_percent_repeat_engraver",
            "Dynamic_align_engraver",
            "Dynamic_engraver",
            "Font_size_engraver",
            "Forbid_line_break_engraver",
            "Glissando_engraver",
            "Grace_auto_beam_engraver",
            "Grace_beam_engraver",
            "Grace_engraver",
            "Grob_pq_engraver",
            "Instrument_switch_engraver",
            "Laissez_vibrer_engraver",
            "Ligature_bracket_engraver",
            "Multi_measure_rest_engraver",
            "Note_head_line_engraver",
            "Note_spacing_engraver",
            "Output_property_engraver",
            "Part_combine_engraver",
            "Percent_repeat_engraver",
            "Phrasing_slur_engraver",
            "Repeat_tie_engraver",
            "Rest_engraver",
            "Rhythmic_column_engraver",
            "Script_column_engraver",
            "Script_engraver",
            "Slash_repeat_engraver",
            "Slur_engraver",
            "Spanner_break_forbid_engraver",
            "Stem_engraver",
            "Tab_note_heads_engraver",
            "Tab_tie_follow_engraver",
            "Text_engraver",
            "Text_spanner_engraver",
            "Tie_engraver",
            "Trill_spanner_engraver",
            "Tuplet_engraver",
            ]),
        },
    "VaticanaStaff": {
        "accepts": set([
            "CueVoice",
            "NullVoice",
            "VaticanaVoice",
            ]),
        "aliases": set([
            "Staff",
            ]),
        "consists": set([
            "Accidental_engraver",
            "Axis_group_engraver",
            "Bar_engraver",
            "Clef_engraver",
            "Collision_engraver",
            "Cue_clef_engraver",
            "Custos_engraver",
            "Dot_column_engraver",
            "Figured_bass_engraver",
            "Figured_bass_position_engraver",
            "Fingering_column_engraver",
            "Font_size_engraver",
            "Grob_pq_engraver",
            "Instrument_name_engraver",
            "Key_engraver",
            "Ledger_line_engraver",
            "Ottava_spanner_engraver",
            "Output_property_engraver",
            "Piano_pedal_align_engraver",
            "Piano_pedal_engraver",
            "Pure_from_neighbor_engraver",
            "Rest_collision_engraver",
            "Script_row_engraver",
            "Separating_line_group_engraver",
            "Staff_collecting_engraver",
            "Staff_symbol_engraver",
            ]),
        "default_child": "VaticanaVoice",
        },
    "VaticanaVoice": {
        "accepts": set([]),
        "aliases": set([
            "Voice",
            ]),
        "consists": set([
            "Arpeggio_engraver",
            "Auto_beam_engraver",
            "Beam_engraver",
            "Bend_engraver",
            "Breathing_sign_engraver",
            "Chord_tremolo_engraver",
            "Cluster_spanner_engraver",
            "Dots_engraver",
            "Double_percent_repeat_engraver",
            "Dynamic_align_engraver",
            "Dynamic_engraver",
            "Episema_engraver",
            "Fingering_engraver",
            "Font_size_engraver",
            "Forbid_line_break_engraver",
            "Glissando_engraver",
            "Grace_auto_beam_engraver",
            "Grace_beam_engraver",
            "Grace_engraver",
            "Grob_pq_engraver",
            "Instrument_switch_engraver",
            "Laissez_vibrer_engraver",
            "Multi_measure_rest_engraver",
            "New_fingering_engraver",
            "Note_head_line_engraver",
            "Note_heads_engraver",
            "Note_spacing_engraver",
            "Output_property_engraver",
            "Part_combine_engraver",
            "Percent_repeat_engraver",
            "Phrasing_slur_engraver",
            "Pitched_trill_engraver",
            "Repeat_tie_engraver",
            "Rest_engraver",
            "Rhythmic_column_engraver",
            "Script_column_engraver",
            "Script_engraver",
            "Slash_repeat_engraver",
            "Spanner_break_forbid_engraver",
            "Text_engraver",
            "Tie_engraver",
            "Trill_spanner_engraver",
            "Tuplet_engraver",
            "Vaticana_ligature_engraver",
            ]),
        },
    "Voice": {
        "accepts": set([]),
        "aliases": set([]),
        "consists": set([
            "Arpeggio_engraver",
            "Auto_beam_engraver",
            "Beam_engraver",
            "Bend_engraver",
            "Breathing_sign_engraver",
            "Chord_tremolo_engraver",
            "Cluster_spanner_engraver",
            "Dots_engraver",
            "Double_percent_repeat_engraver",
            "Dynamic_align_engraver",
            "Dynamic_engraver",
            "Fingering_engraver",
            "Font_size_engraver",
            "Forbid_line_break_engraver",
            "Glissando_engraver",
            "Grace_auto_beam_engraver",
            "Grace_beam_engraver",
            "Grace_engraver",
            "Grob_pq_engraver",
            "Instrument_switch_engraver",
            "Laissez_vibrer_engraver",
            "Ligature_bracket_engraver",
            "Multi_measure_rest_engraver",
            "New_fingering_engraver",
            "Note_head_line_engraver",
            "Note_heads_engraver",
            "Note_spacing_engraver",
            "Output_property_engraver",
            "Part_combine_engraver",
            "Percent_repeat_engraver",
            "Phrasing_slur_engraver",
            "Pitched_trill_engraver",
            "Repeat_tie_engraver",
            "Rest_engraver",
            "Rhythmic_column_engraver",
            "Script_column_engraver",
            "Script_engraver",
            "Slash_repeat_engraver",
            "Slur_engraver",
            "Spanner_break_forbid_engraver",
            "Stem_engraver",
            "Text_engraver",
            "Text_spanner_engraver",
            "Tie_engraver",
            "Trill_spanner_engraver",
            "Tuplet_engraver",
            ]),
        },
    }



import typing
from abjad import enums
from abjad.core.Leaf import Leaf
from abjad.core.MultimeasureRest import MultimeasureRest
from abjad.core.Rest import Rest
from abjad.core.Skip import Skip
from abjad.indicators.Articulation import Articulation
from abjad.indicators.BowContactPoint import BowContactPoint
from abjad.indicators.BowMotionTechnique import BowMotionTechnique
from abjad.lilypondnames.LilyPondGrobOverride import LilyPondGrobOverride
from abjad.scheme import Scheme
from abjad.scheme import SchemeSymbol
from abjad.system.Tag import Tag
from abjad.system.Wrapper import Wrapper
from abjad.top.inspect import inspect
from .Spanner import Spanner


class BowContactSpanner(Spanner):
    r"""
    Bow contact spanner.

    ..  container:: example

        >>> staff = abjad.Staff()
        >>> staff.extend(r"c'4. c'8 \times 2/3 { c'4 c'4 c'4 }")

        >>> leaves = abjad.select(staff).leaves()
        >>> abjad.attach(abjad.BowMotionTechnique('jete'), leaves[0])
        >>> abjad.attach(abjad.BowContactPoint((1, 4)), leaves[0])
        >>> abjad.attach(abjad.BowContactPoint((3, 4)), leaves[1])
        >>> abjad.attach(abjad.BowContactPoint((1, 2)), leaves[2])
        >>> abjad.attach(abjad.BowMotionTechnique('circular'), leaves[3])
        >>> abjad.attach(abjad.BowContactPoint((1, 1)), leaves[3])
        >>> abjad.attach(abjad.BowContactPoint((0, 1)), leaves[4])

        >>> abjad.attach(abjad.Clef('percussion'), leaves[0])
        >>> abjad.override(staff).bar_line.transparent = True
        >>> abjad.override(staff).dots.staff_position = -8
        >>> abjad.override(staff).flag.Y_offset = -8.5
        >>> abjad.override(staff).glissando.bound_details__left__padding = 1.5
        >>> abjad.override(staff).glissando.bound_details__right__padding = 1.5
        >>> abjad.override(staff).glissando.thickness = 2
        >>> abjad.override(staff).script.staff_padding = 3
        >>> abjad.override(staff).staff_symbol.transparent = True
        >>> abjad.override(staff).stem.direction = abjad.Down
        >>> abjad.override(staff).stem.length = 8
        >>> abjad.override(staff).stem.stem_begin_position = -9
        >>> abjad.override(staff).time_signature.stencil = False

        >>> abjad.attach(abjad.BowContactSpanner(), leaves)
        >>> abjad.show(staff) # doctest: +SKIP

        ..  docs::

            >>> abjad.f(staff)
            \new Staff
            \with
            {
                \override BarLine.transparent = ##t
                \override Dots.staff-position = #-8
                \override Flag.Y-offset = #-8.5
                \override Glissando.bound-details.left.padding = #1.5
                \override Glissando.bound-details.right.padding = #1.5
                \override Glissando.thickness = #2
                \override Script.staff-padding = #3
                \override StaffSymbol.transparent = ##t
                \override Stem.direction = #down
                \override Stem.length = #8
                \override Stem.stem-begin-position = #-9
                \override TimeSignature.stencil = ##f
            }
            {
                \once \override Glissando.style = #'dotted-line
                \once \override NoteHead.Y-offset = -1.0
                \once \override NoteHead.stencil = #ly:text-interface::print
                \once \override NoteHead.text = \markup {
                    \center-align
                        \vcenter
                            \fraction
                                1
                                4
                    }
                \clef "percussion"
                c'4.
                ^\downbow
                \glissando
                \once \override NoteHead.Y-offset = 1.0
                \once \override NoteHead.stencil = #ly:text-interface::print
                \once \override NoteHead.text = \markup {
                    \center-align
                        \vcenter
                            \fraction
                                3
                                4
                    }
                c'8
                ^\upbow
                \glissando
                \times 2/3 {
                    \once \override NoteHead.Y-offset = 0.0
                    \once \override NoteHead.stencil = #ly:text-interface::print
                    \once \override NoteHead.text = \markup {
                        \center-align
                            \vcenter
                                \fraction
                                    1
                                    2
                        }
                    c'4
                    ^\downbow
                    \glissando
                    \once \override Glissando.style = #'zigzag
                    \once \override NoteHead.Y-offset = 2.0
                    \once \override NoteHead.stencil = #ly:text-interface::print
                    \once \override NoteHead.text = \markup {
                        \center-align
                            \vcenter
                                \fraction
                                    1
                                    1
                        }
                    c'4
                    ^\upbow
                    \glissando
                    \once \override NoteHead.Y-offset = -2.0
                    \once \override NoteHead.stencil = #ly:text-interface::print
                    \once \override NoteHead.text = \markup {
                        \center-align
                            \vcenter
                                \fraction
                                    0
                                    1
                        }
                    c'4
                }
            }

    ..  container:: example

        Use ``BowContactPoint(None)`` to indicate un-bowed actions, such as
        pizzicato.

        >>> staff = abjad.Staff(r"c'4 c'4 c'4 c'4")

        >>> leaves = staff[:]
        >>> abjad.attach(abjad.BowContactPoint(None), leaves[0])
        >>> abjad.attach(abjad.BowContactPoint((3, 4)), leaves[1])
        >>> abjad.attach(abjad.BowContactPoint((1, 2)), leaves[2])
        >>> abjad.attach(abjad.BowContactPoint(None), leaves[3])

        >>> abjad.attach(abjad.Clef('percussion'), staff[0])
        >>> abjad.override(staff).bar_line.transparent = True
        >>> abjad.override(staff).dots.staff_position = -8
        >>> abjad.override(staff).flag.Y_offset = -8.5
        >>> abjad.override(staff).glissando.bound_details__left__padding = 1.5
        >>> abjad.override(staff).glissando.bound_details__right__padding = 1.5
        >>> abjad.override(staff).glissando.thickness = 2
        >>> abjad.override(staff).script.staff_padding = 3
        >>> abjad.override(staff).staff_symbol.transparent = True
        >>> abjad.override(staff).stem.direction =abjad.Down
        >>> abjad.override(staff).stem.length = 8
        >>> abjad.override(staff).stem.stem_begin_position = -9
        >>> abjad.override(staff).time_signature.stencil = False

        >>> abjad.abjad.attach(abjad.BowContactSpanner(), leaves)
        >>> abjad.show(staff) # doctest: +SKIP

        ..  docs::

            >>> abjad.f(staff)
            \new Staff
            \with
            {
                \override BarLine.transparent = ##t
                \override Dots.staff-position = #-8
                \override Flag.Y-offset = #-8.5
                \override Glissando.bound-details.left.padding = #1.5
                \override Glissando.bound-details.right.padding = #1.5
                \override Glissando.thickness = #2
                \override Script.staff-padding = #3
                \override StaffSymbol.transparent = ##t
                \override Stem.direction = #down
                \override Stem.length = #8
                \override Stem.stem-begin-position = #-9
                \override TimeSignature.stencil = ##f
            }
            {
                \once \override NoteHead.style = #'cross
                \clef "percussion"
                c'4
                \once \override NoteHead.Y-offset = 1.0
                \once \override NoteHead.stencil = #ly:text-interface::print
                \once \override NoteHead.text = \markup {
                    \center-align
                        \vcenter
                            \fraction
                                3
                                4
                    }
                c'4
                ^\upbow
                \glissando
                \once \override NoteHead.Y-offset = 0.0
                \once \override NoteHead.stencil = #ly:text-interface::print
                \once \override NoteHead.text = \markup {
                    \center-align
                        \vcenter
                            \fraction
                                1
                                2
                    }
                c'4
                \once \override NoteHead.style = #'cross
                c'4
            }

    """

    ### CLASS VARIABLES ###

    __slots__ = (
        )

    ### PRIVATE METHODS ###

    def _get_annotations(self, leaf):
        inspector = inspect(leaf)
        bow_contact_point = None
        prototype = BowContactPoint
        if inspector.has_indicator(prototype):
            bow_contact_point = inspector.indicator(prototype)
        bow_motion_technique = None
        prototype = BowMotionTechnique
        if inspector.has_indicator(prototype):
            bow_motion_technique = inspector.indicator(prototype)
        return (
            bow_contact_point,
            bow_motion_technique,
            )

    def _get_lilypond_format_bundle(self, leaf):
        lilypond_format_bundle = self._get_basic_lilypond_format_bundle(leaf)
        indicators = self._get_annotations(leaf)
        bow_contact_point = indicators[0]
        bow_motion_technique = indicators[1]
        #print(leaf)
        if bow_contact_point is None:
            #print('\t', None)
            return lilypond_format_bundle
        if bow_contact_point.contact_point is None:
            #print('\t', 'PIZZ')
            self._make_pizzicato_overrides(lilypond_format_bundle)
            return lilypond_format_bundle
        if len(self) == 1:
            #print('\t', 'ONLY')
            return lilypond_format_bundle
        #print('\t', 'NORM')
        self._make_bow_contact_point_overrides(
            bow_contact_point=bow_contact_point,
            lilypond_format_bundle=lilypond_format_bundle,
            )
        if self._next_leaf_is_bowed(leaf):
            lilypond_format_bundle.after.spanner_starts.append(r'\glissando')
            self._make_bow_direction_change_contributions(
                bow_contact_point=bow_contact_point,
                leaf=leaf,
                lilypond_format_bundle=lilypond_format_bundle,
                )
            self._make_glissando_overrides(
                bow_motion_technique=bow_motion_technique,
                lilypond_format_bundle=lilypond_format_bundle,
                )
        return lilypond_format_bundle

    def _make_bow_contact_point_overrides(
        self,
        bow_contact_point=None,
        lilypond_format_bundle=None,
        ):
        if bow_contact_point is None:
            return
        override_ = LilyPondGrobOverride(
            grob_name='NoteHead',
            once=True,
            property_path='stencil',
            value=Scheme('ly:text-interface::print'),
            )
        string = override_.override_string
        lilypond_format_bundle.grob_overrides.append(string)
        override_ = LilyPondGrobOverride(
            grob_name='NoteHead',
            once=True,
            property_path='text',
            value=bow_contact_point.markup,
            )
        string = override_.override_string
        lilypond_format_bundle.grob_overrides.append(string)
        y_offset = float((4 * bow_contact_point.contact_point) - 2)
        override_ = LilyPondGrobOverride(
            grob_name='NoteHead',
            once=True,
            property_path='Y-offset',
            value=y_offset,
            )
        string = override_.override_string
        lilypond_format_bundle.grob_overrides.append(string)

    def _make_bow_direction_change_contributions(
        self,
        bow_contact_point=None,
        leaf=None,
        lilypond_format_bundle=None,
        ):
        cautionary_change = False
        direction_change = None
        next_leaf = inspect(leaf).leaf(1)
        this_contact_point = bow_contact_point
        if this_contact_point is None:
            return
        next_contact_point = inspect(next_leaf).indicator(BowContactPoint)
        if next_contact_point is None:
            return
        previous_leaf = inspect(leaf).leaf(-1)
        previous_contact_point = None
        if previous_leaf is not None:
            previous_contact_points = inspect(previous_leaf
                ).indicators(BowContactPoint)
            if previous_contact_points:
                previous_contact_point = previous_contact_points[0]
        if (leaf is self[0] or
            previous_contact_point is None or
            previous_contact_point.contact_point is None
            ):
            if this_contact_point < next_contact_point:
                direction_change = enums.Down
            elif next_contact_point < this_contact_point:
                direction_change = enums.Up
        else:
            previous_leaf = inspect(leaf).leaf(-1)
            previous_contact_point = inspect(previous_leaf).indicator(
                BowContactPoint)
            if (previous_contact_point < this_contact_point and
                next_contact_point < this_contact_point):
                direction_change = enums.Up
            elif (this_contact_point < previous_contact_point and
                this_contact_point < next_contact_point):
                direction_change = enums.Down
            elif (this_contact_point == previous_contact_point):
                if this_contact_point < next_contact_point:
                    cautionary_change = True
                    direction_change = enums.Down
                elif next_contact_point < this_contact_point:
                    cautionary_change = True
                    direction_change = enums.Up
        if direction_change is None:
            return
        if cautionary_change:
            if direction_change == enums.Up:
                string = r'^ \parenthesize \upbow'
            elif direction_change == enums.Down:
                string = r'^ \parenthesize \downbow'
        else:
            if direction_change == enums.Up:
                articulation = Articulation('upbow', direction=enums.Up)
            elif direction_change == enums.Down:
                articulation = Articulation('downbow', direction=enums.Up)
            string = str(articulation)
        lilypond_format_bundle.after.articulations.append(string)

    def _make_glissando_overrides(
        self,
        bow_motion_technique=None,
        lilypond_format_bundle=None,
        ):
        if bow_motion_technique is not None:
            style = SchemeSymbol(bow_motion_technique.glissando_style)
            override_ = LilyPondGrobOverride(
                grob_name='Glissando',
                once=True,
                property_path='style',
                value=style,
                )
            string = override_.override_string
            lilypond_format_bundle.grob_overrides.append(string)

    def _make_pizzicato_overrides(
        self,
        lilypond_format_bundle=None,
        ):
        style = SchemeSymbol('cross')
        override_ = LilyPondGrobOverride(
            grob_name='NoteHead',
            once=True,
            property_path='style',
            value=style,
            )
        string = override_.override_string
        lilypond_format_bundle.grob_overrides.append(string)

    def _next_leaf_is_bowed(self, leaf):
        if leaf is self[-1]:
            return False
        prototype = (
            MultimeasureRest,
            Rest,
            Skip,
            )
        next_leaf = inspect(leaf).leaf(1)
        if next_leaf is None or isinstance(next_leaf, prototype):
            return False
        next_contact_point = inspect(next_leaf).indicator(BowContactPoint)
        if next_contact_point is None:
            return False
        elif next_contact_point.contact_point is None:
            return False
        return True



from .NonreducedRatio import NonreducedRatio


class Ratio(NonreducedRatio):
    """
    Ratio.

    ..  container:: example

        Ratio of two numbers:

        >>> abjad.Ratio((2, 4))
        Ratio((1, 2))

    ..  container:: example

        Ratio of three numbers:

        >>> abjad.Ratio((2, 4, 2))
        Ratio((1, 2, 1))

    """

    ### CLASS VARIABLES ###

    __slots__ = ()

    ### INITIALIZER ###

    def __init__(self, numbers=(1, 1)):
        import abjad
        if isinstance(numbers, type(self)):
            numbers = numbers.numbers
        numbers = [int(_) for _ in numbers]
        gcd = abjad.mathtools.greatest_common_divisor(*numbers)
        numbers = [_ // gcd for _ in numbers]
        self._numbers = tuple(numbers)

    ### SPECIAL METHODS ###

    def __eq__(self, argument):
        """
        Is true when ``argument`` equals ratio.

        ..  container:: example

            >>> ratio_1 = abjad.Ratio((1, 2, 1))
            >>> ratio_2 = abjad.Ratio((1, 2, 1))
            >>> ratio_3 = abjad.Ratio((2, 3, 3))

            >>> ratio_1 == ratio_1
            True

            >>> ratio_1 == ratio_2
            True

            >>> ratio_1 == ratio_3
            False

            >>> ratio_2 == ratio_1
            True

            >>> ratio_2 == ratio_2
            True

            >>> ratio_2 == ratio_3
            False

            >>> ratio_3 == ratio_1
            False

            >>> ratio_3 == ratio_2
            False

            >>> ratio_3 == ratio_3
            True

        """
        return super().__eq__(argument)

    def __getitem__(self, argument):
        """
        Gets item or slice identified by ``argument``.

        ..  container:: example

            >>> ratio = abjad.Ratio((2, 4, 2))
            >>> ratio[1]
            2

        Returns integer or tuple.
        """
        if isinstance(argument, slice):
            return tuple(self._numbers.__getitem__(argument))
        return self._numbers.__getitem__(argument)

    def __hash__(self):
        """
        Hashes ratio.

        Required to be explicitly redefined on Python 3 if __eq__ changes.

        Returns integer.
        """
        return super().__hash__()

    def __len__(self):
        """
        Gets length of ratio.

        ..  container:: example

            >>> ratio = abjad.Ratio((2, 4, 2))
            >>> len(ratio)
            3

        Returns integer.
        """
        return len(self._numbers)

    def __str__(self):
        """
        Gets string representation of ratio.

        ..  container:: example

            Ratio of two numbers:

            >>> str(abjad.Ratio((2, 4)))
            '1:2'

        ..  container:: example

            Ratio of three numbers:

            >>> str(abjad.Ratio((2, 4, 2)))
            '1:2:1'

        Returns string.
        """
        numbers = (str(x) for x in self.numbers)
        return ':'.join(numbers)

    ### PUBLIC PROPERTIES ###

    @property
    def multipliers(self):
        """
        Gets multipliers of ratio.

        ..  container:: example

            Ratio of two numbers:

            >>> ratio = abjad.Ratio((2, 4))
            >>> ratio.multipliers
            (Multiplier(1, 3), Multiplier(2, 3))

        ..  container:: example

            Ratio of three numbers:

            >>> ratio = abjad.Ratio((2, 4, 2))
            >>> ratio.multipliers
            (Multiplier(1, 4), Multiplier(1, 2), Multiplier(1, 4))

        Returns tuple of multipliers.
        """
        import abjad
        weight = sum(self.numbers)
        multipliers = [
            abjad.Multiplier((_, weight))
            for _ in self.numbers
            ]
        multipliers = tuple(multipliers)
        return multipliers

    @property
    def numbers(self):
        """
        Gets numbers of ratio.

        ..  container:: example

            Ratio of two numbers:

            >>> ratio = abjad.Ratio((2, 4))
            >>> ratio.numbers
            (1, 2)

        ..  container:: example

            Ratio of three numbers:

            >>> ratio = abjad.Ratio((2, 4, 2))
            >>> ratio.numbers
            (1, 2, 1)

        Set to tuple of two or more numbers.

        Returns tuple of two or more numbers.
        """
        return self._numbers

    @property
    def reciprocal(self):
        """
        Gets reciprocal.

        ..  container:: example

            Gets reciprocal:

            >>> abjad.Ratio((3, 2)).reciprocal
            Ratio((2, 3))

            >>> abjad.Ratio((3, 2, 7)).reciprocal
            Ratio((7, 2, 3))

        Returns new ratio.
        """
        numbers = list(reversed(self.numbers))
        return type(self)(numbers)



import inspect
from abjad import enums
from abjad.system.Signature import Signature
from abjad.top.new import new
from .Segment import Segment


class PitchClassSegment(Segment):
    r"""
    Pitch-class segment.

    ..  container:: example

        Initializes segment with numbered pitch-classes:

        ..  container:: example

            >>> items = [-2, -1.5, 6, 7, -1.5, 7]
            >>> segment = abjad.PitchClassSegment(items=items)
            >>> abjad.show(segment) # doctest: +SKIP

            ..  docs::

                >>> lilypond_file = segment.__illustrate__()
                >>> abjad.f(lilypond_file[abjad.Voice])
                \new Voice
                {
                    bf'8
                    bqf'8
                    fs'8
                    g'8
                    bqf'8
                    g'8
                    \bar "|." %! SCORE1
                    \override Score.BarLine.transparent = ##f
                }

        ..  container:: example expression

            >>> expression = abjad.Expression()
            >>> expression = expression.pitch_class_segment()

            >>> segment = expression([-2, -1.5, 6, 7, -1.5, 7])
            >>> abjad.show(segment) # doctest: +SKIP

            ..  docs::

                >>> lilypond_file = segment.__illustrate__()
                >>> abjad.f(lilypond_file[abjad.Voice])
                \new Voice
                {
                    bf'8
                    bqf'8
                    fs'8
                    g'8
                    bqf'8
                    g'8
                    \bar "|." %! SCORE1
                    \override Score.BarLine.transparent = ##f
                }

    ..  container:: example

        Initializes segment with named pitch-classes:

        ..  container:: example

            >>> items = ['c', 'ef', 'bqs,', 'd']
            >>> segment = abjad.PitchClassSegment(
            ...     items=items,
            ...     item_class=abjad.NamedPitchClass,
            ...     )
            >>> abjad.show(segment) # doctest: +SKIP

            ..  docs::

                >>> lilypond_file = segment.__illustrate__()
                >>> abjad.f(lilypond_file[abjad.Voice])
                \new Voice
                {
                    c'8
                    ef'8
                    bqs'8
                    d'8
                    \bar "|." %! SCORE1
                    \override Score.BarLine.transparent = ##f
                }

        ..  container:: example expression

            >>> expression = abjad.Expression()
            >>> expression = expression.pitch_class_segment(
            ...     item_class=abjad.NamedPitchClass,
            ...     )

            >>> segment = expression(['c', 'ef', 'bqs,', 'd'])
            >>> abjad.show(segment) # doctest: +SKIP

            ..  docs::

                >>> lilypond_file = segment.__illustrate__()
                >>> abjad.f(lilypond_file[abjad.Voice])
                \new Voice
                {
                    c'8
                    ef'8
                    bqs'8
                    d'8
                    \bar "|." %! SCORE1
                    \override Score.BarLine.transparent = ##f
                }

    """

    ### CLASS VARIABLES ###

    __slots__ = ()

    ### INITIALIZER ###

    def __init__(self, items=None, item_class=None):
        if not items and not item_class:
            item_class = self._named_item_class
        Segment.__init__(
            self,
            items=items,
            item_class=item_class,
            )

    ### SPECIAL METHODS ###

    @Signature(
        markup_maker_callback='_make___add___markup',
        string_template_callback='_make___add___string_template',
        )
    def __add__(self, argument):
        r"""
        Adds `argument` to segment.

        ..  container:: example

            Example segments:

            >>> pitch_numbers = [-2, -1.5, 6, 7, -1.5, 7]
            >>> abjad.PitchClassSegment(items=pitch_numbers)
            PitchClassSegment([10, 10.5, 6, 7, 10.5, 7])

            >>> J = abjad.PitchClassSegment(items=pitch_numbers)
            >>> abjad.show(J) # doctest: +SKIP

            >>> pitch_names = ['c', 'ef', 'bqs,', 'd']
            >>> abjad.PitchClassSegment(items=pitch_names)
            PitchClassSegment("c ef bqs d")

            >>> K = abjad.PitchClassSegment(items=pitch_names)
            >>> abjad.show(K) # doctest: +SKIP

        ..  container:: example

            Adds J and K:

            ..  container:: example

                >>> J + K
                PitchClassSegment([10, 10.5, 6, 7, 10.5, 7, 0, 3, 11.5, 2])

                >>> segment = J + K
                >>> abjad.show(segment) # doctest: +SKIP

                ..  docs::

                    >>> lilypond_file = segment.__illustrate__()
                    >>> abjad.f(lilypond_file[abjad.Voice])
                    \new Voice
                    {
                        bf'8
                        bqf'8
                        fs'8
                        g'8
                        bqf'8
                        g'8
                        c'8
                        ef'8
                        bqs'8
                        d'8
                        \bar "|." %! SCORE1
                        \override Score.BarLine.transparent = ##f
                    }

            ..  container:: example expression

                >>> expression_J = abjad.Expression(name='J')
                >>> expression_J = expression_J.pitch_class_segment()
                >>> expression_K = abjad.Expression(name='K')
                >>> expression_K = expression_K.pitch_class_segment()
                >>> expression = expression_J + expression_K

                >>> expression(pitch_numbers, pitch_names)
                PitchClassSegment([10, 10.5, 6, 7, 10.5, 7, 0, 3, 11.5, 2])

                >>> expression.get_string()
                'J + K'

                >>> segment = expression(pitch_numbers, pitch_names)
                >>> markup = expression.get_markup()
                >>> abjad.show(segment, figure_name=markup) # doctest: +SKIP

                ..  docs::

                    >>> lilypond_file = segment.__illustrate__(
                    ...     figure_name=markup,
                    ...     )
                    >>> abjad.f(lilypond_file[abjad.Voice])
                    \new Voice
                    {
                        bf'8
                        ^ \markup {
                            \line
                                {
                                    \bold
                                        J
                                    +
                                    \bold
                                        K
                                }
                            }
                        bqf'8
                        fs'8
                        g'8
                        bqf'8
                        g'8
                        c'8
                        ef'8
                        bqs'8
                        d'8
                        \bar "|." %! SCORE1
                        \override Score.BarLine.transparent = ##f
                    }

        ..  container:: example

            Adds K and J:

            ..  container:: example

                >>> K + J
                PitchClassSegment("c ef bqs d bf bqf fs g bqf g")

                >>> segment = K + J
                >>> abjad.show(segment) # doctest: +SKIP

                ..  docs::

                    >>> lilypond_file = segment.__illustrate__()
                    >>> abjad.f(lilypond_file[abjad.Voice])
                    \new Voice
                    {
                        c'8
                        ef'8
                        bqs'8
                        d'8
                        bf'8
                        bqf'8
                        fs'8
                        g'8
                        bqf'8
                        g'8
                        \bar "|." %! SCORE1
                        \override Score.BarLine.transparent = ##f
                    }

            ..  container:: example expression

                >>> expression_J = abjad.Expression(name='J')
                >>> expression_J = expression_J.pitch_class_segment()
                >>> expression_K = abjad.Expression(name='K')
                >>> expression_K = expression_K.pitch_class_segment()
                >>> expression = expression_K + expression_J

                >>> expression(pitch_names, pitch_numbers)
                PitchClassSegment("c ef bqs d bf bqf fs g bqf g")

                >>> expression.get_string()
                'K + J'

                >>> segment = expression(pitch_names, pitch_numbers)
                >>> markup = expression.get_markup()
                >>> abjad.show(segment, figure_name=markup) # doctest: +SKIP

                ..  docs::

                    >>> lilypond_file = segment.__illustrate__(
                    ...     figure_name=markup,
                    ...     )
                    >>> abjad.f(lilypond_file[abjad.Voice])
                    \new Voice
                    {
                        c'8
                        ^ \markup {
                            \line
                                {
                                    \bold
                                        K
                                    +
                                    \bold
                                        J
                                }
                            }
                        ef'8
                        bqs'8
                        d'8
                        bf'8
                        bqf'8
                        fs'8
                        g'8
                        bqf'8
                        g'8
                        \bar "|." %! SCORE1
                        \override Score.BarLine.transparent = ##f
                    }

        ..  container:: example

            Adds J repeatedly:

            ..  container:: example

                >>> J + J + J
                PitchClassSegment([10, 10.5, 6, 7, 10.5, 7, 10, 10.5, 6, 7, 10.5, 7, 10, 10.5, 6, 7, 10.5, 7])


                >>> segment = J + J + J
                >>> abjad.show(segment) # doctest: +SKIP

                ..  docs::

                    >>> lilypond_file = segment.__illustrate__()
                    >>> abjad.f(lilypond_file[abjad.Voice])
                    \new Voice
                    {
                        bf'8
                        bqf'8
                        fs'8
                        g'8
                        bqf'8
                        g'8
                        bf'8
                        bqf'8
                        fs'8
                        g'8
                        bqf'8
                        g'8
                        bf'8
                        bqf'8
                        fs'8
                        g'8
                        bqf'8
                        g'8
                        \bar "|." %! SCORE1
                        \override Score.BarLine.transparent = ##f
                    }

            ..  container:: example expression

                >>> expression_J = abjad.Expression(name='J')
                >>> expression_J = expression_J.pitch_class_segment()
                >>> expression = expression_J + expression_J + expression_J

                >>> expression(pitch_numbers, pitch_numbers, pitch_numbers)
                PitchClassSegment([10, 10.5, 6, 7, 10.5, 7, 10, 10.5, 6, 7, 10.5, 7, 10, 10.5, 6, 7, 10.5, 7])

                >>> expression.get_string()
                'J + J + J'

                >>> list_ = [pitch_numbers, pitch_numbers, pitch_numbers]
                >>> segment = expression(*list_)
                >>> markup = expression.get_markup()
                >>> abjad.show(segment, figure_name=markup) # doctest: +SKIP

                ..  docs::

                    >>> lilypond_file = segment.__illustrate__(
                    ...     figure_name=markup,
                    ...     )
                    >>> abjad.f(lilypond_file[abjad.Voice])
                    \new Voice
                    {
                        bf'8
                        ^ \markup {
                            \line
                                {
                                    \line
                                        {
                                            \bold
                                                J
                                            +
                                            \bold
                                                J
                                        }
                                    +
                                    \bold
                                        J
                                }
                            }
                        bqf'8
                        fs'8
                        g'8
                        bqf'8
                        g'8
                        bf'8
                        bqf'8
                        fs'8
                        g'8
                        bqf'8
                        g'8
                        bf'8
                        bqf'8
                        fs'8
                        g'8
                        bqf'8
                        g'8
                        \bar "|." %! SCORE1
                        \override Score.BarLine.transparent = ##f
                    }

        ..  container:: example

            Adds transformed segments:

            ..  container:: example

                >>> J.rotate(n=1) + K.rotate(n=2)
                PitchClassSegment([7, 10, 10.5, 6, 7, 10.5, 11.5, 2, 0, 3])

                >>> segment = J.rotate(n=1) + K.rotate(n=2)
                >>> abjad.show(segment) # doctest: +SKIP

                ..  docs::

                    >>> lilypond_file = segment.__illustrate__()
                    >>> abjad.f(lilypond_file[abjad.Voice])
                    \new Voice
                    {
                        g'8
                        bf'8
                        bqf'8
                        fs'8
                        g'8
                        bqf'8
                        bqs'8
                        d'8
                        c'8
                        ef'8
                        \bar "|." %! SCORE1
                        \override Score.BarLine.transparent = ##f
                    }

            ..  container:: example expression

                >>> expression_J = abjad.Expression(name='J')
                >>> expression_J = expression_J.pitch_class_segment()
                >>> expression_J = expression_J.rotate(n=1)
                >>> expression_K = abjad.Expression(name='K')
                >>> expression_K = expression_K.pitch_class_segment()
                >>> expression_K = expression_K.rotate(n=2)
                >>> expression = expression_J + expression_K

                >>> expression(pitch_numbers, pitch_names)
                PitchClassSegment([7, 10, 10.5, 6, 7, 10.5, 11.5, 2, 0, 3])

                >>> expression.get_string()
                'r1(J) + r2(K)'

                >>> segment = expression(pitch_numbers, pitch_names)
                >>> markup = expression.get_markup()
                >>> abjad.show(segment, figure_name=markup) # doctest: +SKIP

                ..  docs::

                    >>> lilypond_file = segment.__illustrate__(
                    ...     figure_name=markup,
                    ...     )
                    >>> abjad.f(lilypond_file[abjad.Voice])
                    \new Voice
                    {
                        g'8
                        ^ \markup {
                            \line
                                {
                                    \concat
                                        {
                                            r
                                            \sub
                                                1
                                            \bold
                                                J
                                        }
                                    +
                                    \concat
                                        {
                                            r
                                            \sub
                                                2
                                            \bold
                                                K
                                        }
                                }
                            }
                        bf'8
                        bqf'8
                        fs'8
                        g'8
                        bqf'8
                        bqs'8
                        d'8
                        c'8
                        ef'8
                        \bar "|." %! SCORE1
                        \override Score.BarLine.transparent = ##f
                    }

        ..  container:: example

            Reverses result:

            ..  container:: example

                >>> segment = J.rotate(n=1) + K.rotate(n=2)
                >>> segment.retrograde()
                PitchClassSegment([3, 0, 2, 11.5, 10.5, 7, 6, 10.5, 10, 7])

                >>> segment = segment.retrograde()
                >>> abjad.show(segment) # doctest: +SKIP

                ..  docs::

                    >>> lilypond_file = segment.__illustrate__()
                    >>> abjad.f(lilypond_file[abjad.Voice])
                    \new Voice
                    {
                        ef'8
                        c'8
                        d'8
                        bqs'8
                        bqf'8
                        g'8
                        fs'8
                        bqf'8
                        bf'8
                        g'8
                        \bar "|." %! SCORE1
                        \override Score.BarLine.transparent = ##f
                    }

            ..  container:: example expression

                >>> expression_J = abjad.Expression(name='J')
                >>> expression_J = expression_J.pitch_class_segment()
                >>> expression_J = expression_J.rotate(n=1)
                >>> expression_K = abjad.Expression(name='K')
                >>> expression_K = expression_K.pitch_class_segment()
                >>> expression_K = expression_K.rotate(n=2)
                >>> expression = expression_J + expression_K
                >>> expression = expression.retrograde()

                >>> expression(pitch_numbers, pitch_names)
                PitchClassSegment([3, 0, 2, 11.5, 10.5, 7, 6, 10.5, 10, 7])

                >>> expression.get_string()
                'R(r1(J) + r2(K))'

                >>> segment = expression(pitch_numbers, pitch_names)
                >>> markup = expression.get_markup()
                >>> abjad.show(segment, figure_name=markup) # doctest: +SKIP

                ..  docs::

                    >>> lilypond_file = segment.__illustrate__(
                    ...     figure_name=markup,
                    ...     )
                    >>> abjad.f(lilypond_file[abjad.Voice])
                    \new Voice
                    {
                        ef'8
                        ^ \markup {
                            \concat
                                {
                                    R
                                    \concat
                                        {
                                            (
                                            \line
                                                {
                                                    \concat
                                                        {
                                                            r
                                                            \sub
                                                                1
                                                            \bold
                                                                J
                                                        }
                                                    +
                                                    \concat
                                                        {
                                                            r
                                                            \sub
                                                                2
                                                            \bold
                                                                K
                                                        }
                                                }
                                            )
                                        }
                                }
                            }
                        c'8
                        d'8
                        bqs'8
                        bqf'8
                        g'8
                        fs'8
                        bqf'8
                        bf'8
                        g'8
                        \bar "|." %! SCORE1
                        \override Score.BarLine.transparent = ##f
                    }

        ..  container:: example expression

            Establishes equivalence:

            >>> expression_J = abjad.Expression(name='J')
            >>> expression_J = expression_J.pitch_class_segment()
            >>> expression_J = expression_J.rotate(n=1)
            >>> expression_K = abjad.Expression(name='K')
            >>> expression_K = expression_K.pitch_class_segment()
            >>> expression_K = expression_K.rotate(n=2)
            >>> expression = expression_J + expression_K
            >>> expression = expression.establish_equivalence(name='Q')

            >>> expression(pitch_numbers, pitch_names)
            PitchClassSegment([7, 10, 10.5, 6, 7, 10.5, 11.5, 2, 0, 3])

            >>> expression.get_string()
            'Q = r1(J) + r2(K)'

            >>> segment = expression(pitch_numbers, pitch_names)
            >>> markup = expression.get_markup()
            >>> abjad.show(segment, figure_name=markup) # doctest: +SKIP

            ..  docs::

                >>> lilypond_file = segment.__illustrate__(
                ...     figure_name=markup,
                ...     )
                >>> abjad.f(lilypond_file[abjad.Voice])
                \new Voice
                {
                    g'8
                    ^ \markup {
                        \line
                            {
                                \bold
                                    Q
                                =
                                \line
                                    {
                                        \concat
                                            {
                                                r
                                                \sub
                                                    1
                                                \bold
                                                    J
                                            }
                                        +
                                        \concat
                                            {
                                                r
                                                \sub
                                                    2
                                                \bold
                                                    K
                                            }
                                    }
                            }
                        }
                    bf'8
                    bqf'8
                    fs'8
                    g'8
                    bqf'8
                    bqs'8
                    d'8
                    c'8
                    ef'8
                    \bar "|." %! SCORE1
                    \override Score.BarLine.transparent = ##f
                }

        ..  container:: example expression

            Transforms equivalence:

            >>> expression = expression.transpose(n=1)

            >>> expression(pitch_numbers, pitch_names)
            PitchClassSegment([8, 11, 11.5, 7, 8, 11.5, 0.5, 3, 1, 4])

            >>> expression.get_string()
            'T1(Q)'

            >>> segment = expression(pitch_numbers, pitch_names)
            >>> markup = expression.get_markup()
            >>> abjad.show(segment, figure_name=markup) # doctest: +SKIP

            ..  docs::

                >>> lilypond_file = segment.__illustrate__(
                ...     figure_name=markup,
                ...     )
                >>> abjad.f(lilypond_file[abjad.Voice])
                \new Voice
                {
                    af'8
                    ^ \markup {
                        \concat
                            {
                                T
                                \sub
                                    1
                                \bold
                                    Q
                            }
                        }
                    b'8
                    bqs'8
                    g'8
                    af'8
                    bqs'8
                    cqs'8
                    ef'8
                    cs'8
                    e'8
                    \bar "|." %! SCORE1
                    \override Score.BarLine.transparent = ##f
                }

        Returns new segment.
        """
        if self._expression:
            return self._update_expresion(inspect.currentframe())
        argument = type(self)(items=argument)
        items = self.items + argument.items
        return type(self)(items=items)

    def __contains__(self, argument):
        r"""
        Is true when pitch-class segment contains `argument`.

        ..  container:: example

            Example segments:

            >>> pitch_numbers = [-2, -1.5, 6, 7, -1.5, 7]
            >>> segment = abjad.PitchClassSegment(items=pitch_numbers)
            >>> segment
            PitchClassSegment([10, 10.5, 6, 7, 10.5, 7])

            >>> abjad.NamedPitch('bf') in segment
            True

            >>> abjad.NamedPitch('cs') in segment
            False

            >>> 'bf' in segment
            True

            >>> 'cs' in segment
            False

            >>> 10 in segment
            True

            >>> 13 in segment
            False

        Returns true or false.
        """
        return super().__contains__(argument)

    def __format__(self, format_specification=''):
        r"""
        Formats segment.

        ..  container:: example

            With numbered pitch-classes:

            >>> pitch_numbers = [-2, -1.5, 6, 7, -1.5, 7]
            >>> J = abjad.PitchClassSegment(items=pitch_numbers)

            >>> abjad.f(J)
            abjad.PitchClassSegment(
                (
                    abjad.NumberedPitchClass(10),
                    abjad.NumberedPitchClass(10.5),
                    abjad.NumberedPitchClass(6),
                    abjad.NumberedPitchClass(7),
                    abjad.NumberedPitchClass(10.5),
                    abjad.NumberedPitchClass(7),
                    ),
                item_class=abjad.NumberedPitchClass,
                )

            >>> abjad.show(J) # doctest: +SKIP

            ..  docs::

                >>> lilypond_file = J.__illustrate__()
                >>> abjad.f(lilypond_file[abjad.Voice])
                \new Voice
                {
                    bf'8
                    bqf'8
                    fs'8
                    g'8
                    bqf'8
                    g'8
                    \bar "|." %! SCORE1
                    \override Score.BarLine.transparent = ##f
                }

        ..  container:: example

            With named pitch-classes:

            >>> pitch_names = ['c', 'ef', 'bqs,', 'd']
            >>> K = abjad.PitchClassSegment(
            ...     items=pitch_names,
            ...     item_class=abjad.NamedPitchClass,
            ...     )

            >>> abjad.f(K)
            abjad.PitchClassSegment(
                (
                    abjad.NamedPitchClass('c'),
                    abjad.NamedPitchClass('ef'),
                    abjad.NamedPitchClass('bqs'),
                    abjad.NamedPitchClass('d'),
                    ),
                item_class=abjad.NamedPitchClass,
                )

            >>> abjad.show(K) # doctest: +SKIP

            ..  docs::

                >>> lilypond_file = K.__illustrate__()
                >>> abjad.f(lilypond_file[abjad.Voice])
                \new Voice
                {
                    c'8
                    ef'8
                    bqs'8
                    d'8
                    \bar "|." %! SCORE1
                    \override Score.BarLine.transparent = ##f
                }

        Returns string.
        """
        return super().__format__(format_specification=format_specification)

    @Signature(
        markup_maker_callback='_make___getitem___markup',
        string_template_callback='_make___getitem___string_template',
        )
    def __getitem__(self, argument):
        r"""
        Gets `argument` from segment.

        ..  container:: example

            Example segment:

            >>> pitch_numbers = [-2, -1.5, 6, 7, -1.5, 7]
            >>> abjad.PitchClassSegment(items=pitch_numbers)
            PitchClassSegment([10, 10.5, 6, 7, 10.5, 7])

            >>> J = abjad.PitchClassSegment(items=pitch_numbers)
            >>> abjad.show(J) # doctest: +SKIP

        ..  container:: example

            Gets item at nonnegative index:

            ..  container:: example

                >>> J[0]
                NumberedPitchClass(10)

            ..  container:: example expression

                >>> expression = abjad.Expression(name='J')
                >>> expression = expression.pitch_class_segment()
                >>> expression = expression[0]

                >>> expression(items=[-2, -1.5, 6, 7, -1.5, 7])
                NumberedPitchClass(10)

                >>> expression.get_string()
                'J[0]'

                >>> markup = expression.get_markup()
                >>> abjad.show(markup) # doctest: +SKIP

                ..  docs::

                    >>> abjad.f(markup)
                    \markup {
                        \concat
                            {
                                \bold
                                    J
                                \sub
                                    0
                            }
                        }

        ..  container:: example

            Gets item at negative index:

            ..  container:: example

                >>> J[-1]
                NumberedPitchClass(7)

            ..  container:: example expression

                >>> expression = abjad.Expression(name='J')
                >>> expression = expression.pitch_class_segment()
                >>> expression = expression[-1]

                >>> expression(items=[-2, -1.5, 6, 7, -1.5, 7])
                NumberedPitchClass(7)

                >>> expression.get_string()
                'J[-1]'

                >>> markup = expression.get_markup()
                >>> abjad.show(markup) # doctest: +SKIP

                ..  docs::

                    >>> abjad.f(markup)
                    \markup {
                        \concat
                            {
                                \bold
                                    J
                                \sub
                                    -1
                            }
                        }

        ..  container:: example

            Gets slice:

            ..  container:: example

                >>> J[:4]
                PitchClassSegment([10, 10.5, 6, 7])

                >>> segment = J[:4]
                >>> abjad.show(segment) # doctest: +SKIP

                ..  docs::

                    >>> lilypond_file = segment.__illustrate__()
                    >>> abjad.f(lilypond_file[abjad.Voice])
                    \new Voice
                    {
                        bf'8
                        bqf'8
                        fs'8
                        g'8
                        \bar "|." %! SCORE1
                        \override Score.BarLine.transparent = ##f
                    }

            ..  container:: example expression

                >>> expression = abjad.Expression(name='J')
                >>> expression = expression.pitch_class_segment()
                >>> expression = expression[:4]

                >>> expression(items=[-2, -1.5, 6, 7, -1.5, 7])
                PitchClassSegment([10, 10.5, 6, 7])

                >>> expression.get_string()
                'J[:4]'

                >>> segment = expression(items=[-2, -1.5, 6, 7, -1.5, 7])
                >>> markup = expression.get_markup()
                >>> abjad.show(segment, figure_name=markup) # doctest: +SKIP

                ..  docs::

                    >>> lilypond_file = segment.__illustrate__(
                    ...     figure_name=markup,
                    ...     )
                    >>> abjad.f(lilypond_file[abjad.Voice])
                    \new Voice
                    {
                        bf'8
                        ^ \markup {
                            \concat
                                {
                                    \bold
                                        J
                                    \sub
                                        [:4]
                                }
                            }
                        bqf'8
                        fs'8
                        g'8
                        \bar "|." %! SCORE1
                        \override Score.BarLine.transparent = ##f
                    }

        ..  container:: example

            Gets retrograde of slice:

            ..  container:: example

                >>> J[:4].retrograde()
                PitchClassSegment([7, 6, 10.5, 10])

                >>> segment = J[:4].retrograde()
                >>> abjad.show(segment) # doctest: +SKIP

                ..  docs::

                    >>> lilypond_file = segment.__illustrate__()
                    >>> abjad.f(lilypond_file[abjad.Voice])
                    \new Voice
                    {
                        g'8
                        fs'8
                        bqf'8
                        bf'8
                        \bar "|." %! SCORE1
                        \override Score.BarLine.transparent = ##f
                    }

            ..  container:: example expression

                >>> expression = abjad.Expression(name='J')
                >>> expression = expression.pitch_class_segment()
                >>> expression = expression[:4]
                >>> expression = expression.retrograde()

                >>> expression(items=[-2, -1.5, 6, 7, -1.5, 7])
                PitchClassSegment([7, 6, 10.5, 10])

                >>> expression.get_string()
                'R(J[:4])'

                >>> segment = expression(items=[-2, -1.5, 6, 7, -1.5, 7])
                >>> markup = expression.get_markup()
                >>> abjad.show(segment, figure_name=markup) # doctest: +SKIP

                ..  docs::

                    >>> lilypond_file = segment.__illustrate__(
                    ...     figure_name=markup,
                    ...     )
                    >>> abjad.f(lilypond_file[abjad.Voice])
                    \new Voice
                    {
                        g'8
                        ^ \markup {
                            \concat
                                {
                                    R
                                    \concat
                                        {
                                            \bold
                                                J
                                            \sub
                                                [:4]
                                        }
                                }
                            }
                        fs'8
                        bqf'8
                        bf'8
                        \bar "|." %! SCORE1
                        \override Score.BarLine.transparent = ##f
                    }

        ..  container:: example

            Gets slice of retrograde:

            ..  container:: example

                >>> J.retrograde()[:4]
                PitchClassSegment([7, 10.5, 7, 6])

                >>> segment = J.retrograde()[:4]
                >>> abjad.show(segment) # doctest: +SKIP

                ..  docs::

                    >>> lilypond_file = segment.__illustrate__()
                    >>> abjad.f(lilypond_file[abjad.Voice])
                    \new Voice
                    {
                        g'8
                        bqf'8
                        g'8
                        fs'8
                        \bar "|." %! SCORE1
                        \override Score.BarLine.transparent = ##f
                    }

            ..  container:: example expression

                >>> expression = abjad.Expression(name='J')
                >>> expression = expression.pitch_class_segment()
                >>> expression = expression.retrograde()
                >>> expression = expression[:4]

                >>> expression(items=[-2, -1.5, 6, 7, -1.5, 7])
                PitchClassSegment([7, 10.5, 7, 6])

                >>> expression.get_string()
                'R(J)[:4]'

                >>> segment = expression(items=[-2, -1.5, 6, 7, -1.5, 7])
                >>> markup = expression.get_markup()
                >>> abjad.show(segment, figure_name=markup) # doctest: +SKIP

                ..  docs::

                    >>> lilypond_file = segment.__illustrate__(
                    ...     figure_name=markup,
                    ...     )
                    >>> abjad.f(lilypond_file[abjad.Voice])
                    \new Voice
                    {
                        g'8
                        ^ \markup {
                            \concat
                                {
                                    \concat
                                        {
                                            (
                                            \concat
                                                {
                                                    R
                                                    \bold
                                                        J
                                                }
                                            )
                                        }
                                    \sub
                                        [:4]
                                }
                            }
                        bqf'8
                        g'8
                        fs'8
                        \bar "|." %! SCORE1
                        \override Score.BarLine.transparent = ##f
                    }

        ..  container:: example

            Returns pitch-class or pitch-class segment:

            >>> isinstance(segment, abjad.PitchClassSegment)
            True

        """
        if self._expression:
            return self._update_expression(
                inspect.currentframe(),
                precedence=100,
                )
        return super().__getitem__(argument)

    def __illustrate__(self, expression_markup_direction=enums.Up, **keywords):
        r"""
        Illustrates segment.

        ..  container:: example

            Illustrates numbered segment:

            >>> items = [-2, -1.5, 6, 7, -1.5, 7]
            >>> segment = abjad.PitchClassSegment(items=items)
            >>> abjad.show(segment) # doctest: +SKIP

            ..  docs::

                >>> lilypond_file = segment.__illustrate__()
                >>> abjad.f(lilypond_file[abjad.Voice])
                \new Voice
                {
                    bf'8
                    bqf'8
                    fs'8
                    g'8
                    bqf'8
                    g'8
                    \bar "|." %! SCORE1
                    \override Score.BarLine.transparent = ##f
                }

        ..  container:: example

            Illustrates named segment:

            >>> items = ['c', 'ef', 'bqs,', 'd']
            >>> segment = abjad.PitchClassSegment(
            ...     items=items,
            ...     item_class=abjad.NumberedPitchClass,
            ...     )
            >>> abjad.show(segment) # doctest: +SKIP

            ..  docs::

                >>> lilypond_file = segment.__illustrate__()
                >>> abjad.f(lilypond_file[abjad.Voice])
                \new Voice
                {
                    c'8
                    ef'8
                    bqs'8
                    d'8
                    \bar "|." %! SCORE1
                    \override Score.BarLine.transparent = ##f
                }

        ..  container:: example

            Returns LilyPond file:

            >>> prototype = abjad.LilyPondFile
            >>> isinstance(segment.__illustrate__(), prototype)
            True

        """
        return super().__illustrate__(
            expression_markup_direction=expression_markup_direction,
            **keywords
            )

    def __mul__(self, n):
        r"""
        Multiplies pitch-class segment by `n`.

        ..  container:: example

            >>> items = [-2, -1.5, 6, 7, -1.5, 7]
            >>> 2 * abjad.PitchClassSegment(items=items)
            PitchClassSegment([10, 10.5, 6, 7, 10.5, 7, 10, 10.5, 6, 7, 10.5, 7])

        Returns new pitch-class segment.
        """
        return super().__mul__(n)

    def __repr__(self):
        r"""
        Gets interpreter representation.

        ..  container:: example

            Interpreter representation:

            >>> items = [-2, -1.5, 6, 7, -1.5, 7]
            >>> abjad.PitchClassSegment(items=items)
            PitchClassSegment([10, 10.5, 6, 7, 10.5, 7])

        Returns string.
        """
        import abjad
        if self.item_class is abjad.NamedPitchClass:
            contents = ' '.join([str(_) for _ in self])
            contents = '"' + contents + '"'
        else:
            contents = ', '.join([str(_) for _ in self])
            contents = '[' + contents + ']'
        return '{}({})'.format(type(self).__name__, contents)

    def __rmul__(self, n):
        r"""
        Multiplies `n` by pitch-class segment.

        ..  container:: example

            >>> items = [-2, -1.5, 6, 7, -1.5, 7]
            >>> abjad.PitchClassSegment(items=items) * 2
            PitchClassSegment([10, 10.5, 6, 7, 10.5, 7, 10, 10.5, 6, 7, 10.5, 7])

        Returns new pitch-class segment.
        """
        return super().__rmul__(n)

    def __str__(self):
        r"""
        Gets string representation of pitch-class segment.

        ..  container::

            Gets string represenation of numbered pitch class:

            >>> segment = abjad.PitchClassSegment([-2, -1.5, 6, 7, -1.5, 7])
            >>> str(segment)
            'PC<10, 10.5, 6, 7, 10.5, 7>'

        ..  container::

            Gets string represenation of named pitch class:

            >>> segment = abjad.PitchClassSegment(
            ...     items=[-2, -1.5, 6, 7, -1.5, 7],
            ...     item_class=abjad.NamedPitchClass,
            ...     )
            >>> str(segment)
            'PC<bf bqf fs g bqf g>'

        Returns string.
        """
        import abjad
        items = [str(_) for _ in self]
        separator = ' '
        if self.item_class is abjad.NumberedPitchClass:
            separator = ', '
        return 'PC<{}>'.format(separator.join(items))

    ### PRIVATE PROPERTIES ###

    @property
    def _named_item_class(self):
        import abjad
        return abjad.NamedPitchClass

    @property
    def _numbered_item_class(self):
        import abjad
        return abjad.NumberedPitchClass

    @property
    def _parent_item_class(self):
        import abjad
        return abjad.PitchClass

    ### PRIVATE METHODS ###

    def _get_padded_string(self, width=2):
        string = super()._get_padded_string(width=width)
        return 'PC<' + string[1:-1] + '>'

    def _is_equivalent_under_transposition(self, argument):
        import abjad
        if not isinstance(argument, type(self)):
            return False
        if not len(self) == len(argument):
            return False
        difference = -(
            abjad.NamedPitch((argument[0].name, 4)) -
            abjad.NamedPitch((self[0].name, 4))
            )
        new_pitch_classes = (x + difference for x in self)
        new_pitch_classes = new(self, items=new_pitch_classes)
        return argument == new_pitch_classes

    @staticmethod
    def _make_rotate_method_name(n=0, stravinsky=False):
        if stravinsky:
            return 'rs'
        return 'r'

    def _transpose_to_zero(self):
        numbers = [_.number for _ in self]
        first_number = self[0].number
        numbers = [pc.number - first_number for pc in self]
        pcs = [_ % 12 for _ in numbers]
        return type(self)(items=pcs, item_class=self.item_class)

    def _update_expression(self, frame, precedence=None):
        import abjad
        callback = abjad.Expression._frame_to_callback(
            frame,
            precedence=precedence,
            )
        return self._expression.append_callback(callback)

    ### PUBLIC PROPERTIES ###

    @property
    def item_class(self):
        r"""
        Gets item class of segment.

        ..  container:: example

            Gets item class of numbered segment:

            >>> items = [-2, -1.5, 6, 7, -1.5, 7]
            >>> segment = abjad.PitchClassSegment(items=items)
            >>> abjad.show(segment) # doctest: +SKIP

            >>> segment.item_class
            <class 'abjad.pitch.NumberedPitchClass.NumberedPitchClass'>

        ..  container:: example


            Gets item class of named segment:

            >>> items = ['c', 'ef', 'bqs,', 'd']
            >>> segment = abjad.PitchClassSegment(
            ...     items=items,
            ...     item_class=abjad.NamedPitchClass,
            ...     )
            >>> abjad.show(segment) # doctest: +SKIP

            >>> segment.item_class
            <class 'abjad.pitch.NamedPitchClass.NamedPitchClass'>

        ..  container:: example

            Returns class:

            >>> type(segment.item_class)
            <class 'abc.ABCMeta'>

        """
        return super().item_class

    @property
    def items(self):
        r"""
        Gets items in segment.

        ..  container:: example

            ..  container:: example

                Initializes items positionally:

                >>> items = [-2, -1.5, 6, 7, -1.5, 7]
                >>> segment = abjad.PitchClassSegment(items)
                >>> for item in segment.items:
                ...     item
                ...
                NumberedPitchClass(10)
                NumberedPitchClass(10.5)
                NumberedPitchClass(6)
                NumberedPitchClass(7)
                NumberedPitchClass(10.5)
                NumberedPitchClass(7)

                Initializes items from keyword:

                >>> items = [-2, -1.5, 6, 7, -1.5, 7]
                >>> segment = abjad.PitchClassSegment(items=items)
                >>> for item in segment.items:
                ...     item
                NumberedPitchClass(10)
                NumberedPitchClass(10.5)
                NumberedPitchClass(6)
                NumberedPitchClass(7)
                NumberedPitchClass(10.5)
                NumberedPitchClass(7)

            ..  container:: example expression

                Initializes items positionally:

                >>> expression = abjad.Expression()
                >>> expression = expression.pitch_class_segment()

                >>> items = [-2, -1.5, 6, 7, -1.5, 7]
                >>> segment = expression(items)
                >>> for item in segment.items:
                ...     item
                NumberedPitchClass(10)
                NumberedPitchClass(10.5)
                NumberedPitchClass(6)
                NumberedPitchClass(7)
                NumberedPitchClass(10.5)
                NumberedPitchClass(7)

                Initializes items from keyword:

                >>> expression = abjad.Expression()
                >>> expression = expression.pitch_class_segment()

                >>> items = [-2, -1.5, 6, 7, -1.5, 7]
                >>> segment = expression(items=items)
                >>> for item in segment.items:
                ...     item
                NumberedPitchClass(10)
                NumberedPitchClass(10.5)
                NumberedPitchClass(6)
                NumberedPitchClass(7)
                NumberedPitchClass(10.5)
                NumberedPitchClass(7)

        ..  container:: example

            Returns list:

            >>> isinstance(segment.items, list)
            True

        """
        return super().items

    ### PUBLIC METHODS ###

    def count(self, item):
        """
        Counts `item` in segment.

        ..  container:: example

            Example segment:

            >>> items = [-2, -1.5, 6, 7, -1.5, 7]
            >>> segment = abjad.PitchClassSegment(items=items)
            >>> abjad.show(segment) # doctest: +SKIP

        ..  container:: example

            Counts existing item in segment:

            >>> segment.count(-1.5)
            2

        ..  container:: example

            Counts nonexisting item in segment:

            >>> segment.count('text')
            0

        ..  container:: example

            Returns nonnegative integer:

            >>> isinstance(segment.count('text'), int)
            True

        """
        return super().count(item)

    @classmethod
    def from_selection(class_, selection, item_class=None):
        """
        Initializes segment from `selection`.

        ..  container:: example

            Initializes from selection:

            >>> staff_1 = abjad.Staff("c'4 <d' fs' a'>4 b2")
            >>> staff_2 = abjad.Staff("c4. r8 g2")
            >>> staff_group = abjad.StaffGroup([staff_1, staff_2])
            >>> abjad.show(staff_group) # doctest: +SKIP

            >>> selection = abjad.select((staff_1, staff_2))
            >>> segment = abjad.PitchClassSegment.from_selection(selection)
            >>> abjad.show(segment) # doctest: +SKIP

        ..  container:: example

            Returns pitch-class segment:

            >>> segment
            PitchClassSegment("c d fs a b c g")

        """
        import abjad
        pitch_segment = abjad.PitchSegment.from_selection(selection)
        return class_(
            items=pitch_segment,
            item_class=item_class,
            )

    def has_duplicates(self):
        """
        Is true when segment contains duplicate items.

        ..  container:: example

            Has duplicates:

            >>> items = [-2, -1.5, 6, 7, -1.5, 7]
            >>> segment = abjad.PitchClassSegment(items=items)
            >>> abjad.show(segment) # doctest: +SKIP

            >>> segment.has_duplicates()
            True

        ..  container:: example

            Has no duplicates:

            >>> items = "c d e f g a b"
            >>> segment = abjad.PitchClassSegment(items=items)
            >>> abjad.show(segment) # doctest: +SKIP

            >>> segment.has_duplicates()
            False

        Returns true or false.
        """
        import abjad
        return len(abjad.PitchClassSet(self)) < len(self)

    def index(self, item):
        """
        Gets index of `item` in segment.

        ..  container:: example

            Example segment:

            >>> items = [-2, -1.5, 6, 7, -1.5, 7]
            >>> segment = abjad.PitchClassSegment(items=items)
            >>> abjad.show(segment) # doctest: +SKIP

        ..  container:: example

            Gets index of first item in segment:

            >>> segment.index(-2)
            0

        ..  container:: example

            Gets index of second item in segment:

            >>> segment.index(-1.5)
            1

        ..  container:: example

            Returns nonnegative integer:

            >>> isinstance(segment.index(-1.5), int)
            True

        """
        return super().index(item)

    @Signature(
        is_operator=True,
        method_name='I',
        subscript='axis',
        )
    def invert(self, axis=None):
        r"""
        Inverts segment.

        ..  container:: example

            Example segment:

            >>> items = [-2, -1.5, 6, 7, -1.5, 7]
            >>> J = abjad.PitchClassSegment(items=items)
            >>> J
            PitchClassSegment([10, 10.5, 6, 7, 10.5, 7])

            >>> abjad.show(J) # doctest: +SKIP

        ..  container:: example

            Inverts segment:

            ..  container:: example

                >>> J.invert()
                PitchClassSegment([2, 1.5, 6, 5, 1.5, 5])

                >>> segment = J.invert()
                >>> abjad.show(segment) # doctest: +SKIP

                ..  docs::

                    >>> lilypond_file = segment.__illustrate__()
                    >>> abjad.f(lilypond_file[abjad.Voice])
                    \new Voice
                    {
                        d'8
                        dqf'8
                        fs'8
                        f'8
                        dqf'8
                        f'8
                        \bar "|." %! SCORE1
                        \override Score.BarLine.transparent = ##f
                    }

            ..  container:: example expression

                >>> expression = abjad.Expression(name='J')
                >>> expression = expression.pitch_class_segment()
                >>> expression = expression.invert()

                >>> expression([-2, -1.5, 6, 7, -1.5, 7])
                PitchClassSegment([2, 1.5, 6, 5, 1.5, 5])

                >>> expression.get_string()
                'I(J)'

                >>> segment = expression([-2, -1.5, 6, 7, -1.5, 7])
                >>> markup = expression.get_markup()
                >>> abjad.show(segment, figure_name=markup) # doctest: +SKIP

                ..  docs::

                    >>> lilypond_file = segment.__illustrate__(
                    ...     figure_name=markup,
                    ...     )
                    >>> abjad.f(lilypond_file[abjad.Voice])
                    \new Voice
                    {
                        d'8
                        ^ \markup {
                            \concat
                                {
                                    I
                                    \bold
                                        J
                                }
                            }
                        dqf'8
                        fs'8
                        f'8
                        dqf'8
                        f'8
                        \bar "|." %! SCORE1
                        \override Score.BarLine.transparent = ##f
                    }

        ..  container:: example

            Inverts inversion of segment:

            ..  container:: example

                >>> J.invert().invert()
                PitchClassSegment([10, 10.5, 6, 7, 10.5, 7])

                >>> segment = J.invert().invert()
                >>> abjad.show(segment) # doctest: +SKIP

                ..  docs::

                    >>> lilypond_file = segment.__illustrate__()
                    >>> abjad.f(lilypond_file[abjad.Voice])
                    \new Voice
                    {
                        bf'8
                        bqf'8
                        fs'8
                        g'8
                        bqf'8
                        g'8
                        \bar "|." %! SCORE1
                        \override Score.BarLine.transparent = ##f
                    }

                >>> segment == J
                True

            ..  container:: example expression

                >>> expression = abjad.Expression(name='J')
                >>> expression = expression.pitch_class_segment()
                >>> expression = expression.invert()
                >>> expression = expression.invert()

                >>> expression([-2, -1.5, 6, 7, -1.5, 7])
                PitchClassSegment([10, 10.5, 6, 7, 10.5, 7])

                >>> expression.get_string()
                'I(I(J))'

                >>> segment = expression([-2, -1.5, 6, 7, -1.5, 7])
                >>> markup = expression.get_markup()
                >>> abjad.show(segment, figure_name=markup) # doctest: +SKIP

                ..  docs::

                    >>> lilypond_file = segment.__illustrate__(
                    ...     figure_name=markup,
                    ...     )
                    >>> abjad.f(lilypond_file[abjad.Voice])
                    \new Voice
                    {
                        bf'8
                        ^ \markup {
                            \concat
                                {
                                    I
                                    \concat
                                        {
                                            I
                                            \bold
                                                J
                                        }
                                }
                            }
                        bqf'8
                        fs'8
                        g'8
                        bqf'8
                        g'8
                        \bar "|." %! SCORE1
                        \override Score.BarLine.transparent = ##f
                    }

        ..  container:: example

            Returns pitch-class segment:

            >>> isinstance(segment, abjad.PitchClassSegment)
            True

        """
        if self._expression:
            return self._update_expression(inspect.currentframe())
        items = [_.invert(axis=axis) for _ in self]
        return type(self)(items=items)

    def make_notes(self, n=None, written_duration=None):
        r"""
        Makes first `n` notes in segment.

        ..  container:: example

            Example segment:

            >>> items = [2, 4.5, 6, 11, 4.5, 10]
            >>> segment = abjad.PitchClassSegment(items=items)
            >>> abjad.show(segment) # doctest: +SKIP

        ..  container:: example

            Makes eighth notes:

            >>> notes = segment.make_notes()
            >>> staff = abjad.Staff(notes)
            >>> abjad.show(staff) # doctest: +SKIP

            ..  docs::

                >>> abjad.f(staff)
                \new Staff
                {
                    d'8
                    eqs'8
                    fs'8
                    b'8
                    eqs'8
                    bf'8
                }

        ..  container:: example

            Makes notes with nonassignable durations:

            >>> notes = segment.make_notes(4, abjad.Duration(5, 16))
            >>> staff = abjad.Staff(notes)
            >>> time_signature = abjad.TimeSignature((5, 4))
            >>> abjad.attach(time_signature, staff[0])
            >>> abjad.show(staff) # doctest: +SKIP

            ..  docs::

                >>> abjad.f(staff)
                \new Staff
                {
                    \time 5/4
                    d'4
                    ~
                    d'16
                    eqs'4
                    ~
                    eqs'16
                    fs'4
                    ~
                    fs'16
                    b'4
                    ~
                    b'16
                }

        Interprets none-valued `n` equal to length of segment.

        Interprets none-valued `written_duration` equal to 1/8.

        ..  container:: example

            Returns selection:

            >>> isinstance(segment.make_notes(), abjad.Selection)
            True

        """
        import abjad
        n = n or len(self)
        written_duration = written_duration or abjad.Duration(1, 8)
        maker = abjad.NoteMaker()
        result = maker([0] * n, [written_duration])
        logical_ties = abjad.iterate(result).logical_ties()
        for i, logical_tie in enumerate(logical_ties):
            pitch_class = abjad.NamedPitchClass(self[i % len(self)])
            pitch = abjad.NamedPitch((pitch_class.name, 4))
            for note in logical_tie:
                note.written_pitch = pitch
        return result

    @Signature(
        is_operator=True,
        method_name='M',
        subscript='n',
        )
    def multiply(self, n=1):
        r"""
        Multiplies pitch-classes in segment by `n`.

        ..  container:: example

            Example segment:

            >>> items = [-2, -1.5, 6, 7, -1.5, 7]
            >>> J = abjad.PitchClassSegment(items=items)
            >>> J
            PitchClassSegment([10, 10.5, 6, 7, 10.5, 7])

            >>> abjad.show(J) # doctest: +SKIP

        ..  container:: example

            Multiplies pitch-classes in segment by 1:

            ..  container:: example

                >>> J.multiply(n=1)
                PitchClassSegment([10, 10.5, 6, 7, 10.5, 7])

                >>> segment = J.multiply(n=1)
                >>> abjad.show(segment) # doctest: +SKIP

                ..  docs::

                    >>> lilypond_file = segment.__illustrate__()
                    >>> abjad.f(lilypond_file[abjad.Voice])
                    \new Voice
                    {
                        bf'8
                        bqf'8
                        fs'8
                        g'8
                        bqf'8
                        g'8
                        \bar "|." %! SCORE1
                        \override Score.BarLine.transparent = ##f
                    }

            ..  container:: example expression

                >>> expression = abjad.Expression(name='J')
                >>> expression = expression.pitch_class_segment()
                >>> expression = expression.multiply(n=1)

                >>> expression([-2, -1.5, 6, 7, -1.5, 7])
                PitchClassSegment([10, 10.5, 6, 7, 10.5, 7])

                >>> expression.get_string()
                'M1(J)'

                >>> segment = expression([-2, -1.5, 6, 7, -1.5, 7])
                >>> markup = expression.get_markup()
                >>> abjad.show(segment, figure_name=markup) # doctest: +SKIP

                ..  docs::

                    >>> lilypond_file = segment.__illustrate__(
                    ...     figure_name=markup,
                    ...     )
                    >>> abjad.f(lilypond_file[abjad.Voice])
                    \new Voice
                    {
                        bf'8
                        ^ \markup {
                            \concat
                                {
                                    M
                                    \sub
                                        1
                                    \bold
                                        J
                                }
                            }
                        bqf'8
                        fs'8
                        g'8
                        bqf'8
                        g'8
                        \bar "|." %! SCORE1
                        \override Score.BarLine.transparent = ##f
                    }

        ..  container:: example

            Multiplies pitch-classes in segment by 5:

            ..  container:: example

                >>> J.multiply(n=5)
                PitchClassSegment([2, 4.5, 6, 11, 4.5, 11])

                >>> segment = J.multiply(n=5)
                >>> abjad.show(segment) # doctest: +SKIP

                ..  docs::

                    >>> lilypond_file = segment.__illustrate__()
                    >>> abjad.f(lilypond_file[abjad.Voice])
                    \new Voice
                    {
                        d'8
                        eqs'8
                        fs'8
                        b'8
                        eqs'8
                        b'8
                        \bar "|." %! SCORE1
                        \override Score.BarLine.transparent = ##f
                    }

            ..  container:: example expression

                >>> expression = abjad.Expression(name='J')
                >>> expression = expression.pitch_class_segment()
                >>> expression = expression.multiply(n=5)

                >>> expression([-2, -1.5, 6, 7, -1.5, 7])
                PitchClassSegment([2, 4.5, 6, 11, 4.5, 11])

                >>> expression.get_string()
                'M5(J)'

                >>> segment = expression([-2, -1.5, 6, 7, -1.5, 7])
                >>> markup = expression.get_markup()
                >>> abjad.show(segment, figure_name=markup) # doctest: +SKIP

                ..  docs::

                    >>> lilypond_file = segment.__illustrate__(
                    ...     figure_name=markup,
                    ...     )
                    >>> abjad.f(lilypond_file[abjad.Voice])
                    \new Voice
                    {
                        d'8
                        ^ \markup {
                            \concat
                                {
                                    M
                                    \sub
                                        5
                                    \bold
                                        J
                                }
                            }
                        eqs'8
                        fs'8
                        b'8
                        eqs'8
                        b'8
                        \bar "|." %! SCORE1
                        \override Score.BarLine.transparent = ##f
                    }

        ..  container:: example

            Multiplies pitch-classes in segment by 7:

            ..  container:: example

                >>> J.multiply(n=7)
                PitchClassSegment([10, 1.5, 6, 1, 1.5, 1])

                >>> segment = J.multiply(n=7)
                >>> abjad.show(segment) # doctest: +SKIP

                ..  docs::

                    >>> lilypond_file = segment.__illustrate__()
                    >>> abjad.f(lilypond_file[abjad.Voice])
                    \new Voice
                    {
                        bf'8
                        dqf'8
                        fs'8
                        cs'8
                        dqf'8
                        cs'8
                        \bar "|." %! SCORE1
                        \override Score.BarLine.transparent = ##f
                    }

            ..  container:: example expression

                >>> expression = abjad.Expression(name='J')
                >>> expression = expression.pitch_class_segment()
                >>> expression = expression.multiply(n=7)

                >>> expression([-2, -1.5, 6, 7, -1.5, 7])
                PitchClassSegment([10, 1.5, 6, 1, 1.5, 1])

                >>> expression.get_string()
                'M7(J)'

                >>> segment = expression([-2, -1.5, 6, 7, -1.5, 7])
                >>> markup = expression.get_markup()
                >>> abjad.show(segment, figure_name=markup) # doctest: +SKIP

                ..  docs::

                    >>> lilypond_file = segment.__illustrate__(
                    ...     figure_name=markup,
                    ...     )
                    >>> abjad.f(lilypond_file[abjad.Voice])
                    \new Voice
                    {
                        bf'8
                        ^ \markup {
                            \concat
                                {
                                    M
                                    \sub
                                        7
                                    \bold
                                        J
                                }
                            }
                        dqf'8
                        fs'8
                        cs'8
                        dqf'8
                        cs'8
                        \bar "|." %! SCORE1
                        \override Score.BarLine.transparent = ##f
                    }

        ..  container:: example

            Multiplies pitch-classes in segment by 11:

            ..  container:: example

                >>> segment = J.multiply(n=11)
                >>> segment
                PitchClassSegment([2, 7.5, 6, 5, 7.5, 5])

                >>> abjad.show(segment) # doctest: +SKIP

                ..  docs::

                    >>> lilypond_file = segment.__illustrate__()
                    >>> abjad.f(lilypond_file[abjad.Voice])
                    \new Voice
                    {
                        d'8
                        gqs'8
                        fs'8
                        f'8
                        gqs'8
                        f'8
                        \bar "|." %! SCORE1
                        \override Score.BarLine.transparent = ##f
                    }

            ..  container:: example expression

                >>> expression = abjad.Expression(name='J')
                >>> expression = expression.pitch_class_segment()
                >>> expression = expression.multiply(n=11)

                >>> expression([-2, -1.5, 6, 7, -1.5, 7])
                PitchClassSegment([2, 7.5, 6, 5, 7.5, 5])

                >>> expression.get_string()
                'M11(J)'

                >>> segment = expression([-2, -1.5, 6, 7, -1.5, 7])
                >>> markup = expression.get_markup()
                >>> abjad.show(segment, figure_name=markup) # doctest: +SKIP

                ..  docs::

                    >>> lilypond_file = segment.__illustrate__(
                    ...     figure_name=markup,
                    ...     )
                    >>> abjad.f(lilypond_file[abjad.Voice])
                    \new Voice
                    {
                        d'8
                        ^ \markup {
                            \concat
                                {
                                    M
                                    \sub
                                        11
                                    \bold
                                        J
                                }
                            }
                        gqs'8
                        fs'8
                        f'8
                        gqs'8
                        f'8
                        \bar "|." %! SCORE1
                        \override Score.BarLine.transparent = ##f
                    }

        ..  container:: example

            Returns pitch-class segment:

            >>> isinstance(segment, abjad.PitchClassSegment)
            True

        """
        import abjad
        if self._expression:
            return self._update_expression(inspect.currentframe())
        items = [abjad.NumberedPitchClass(_) for _ in self]
        items = [_.multiply(n) for _ in items]
        return type(self)(items=items)

    @Signature()
    def permute(self, row=None):
        r"""
        Permutes segment by twelve-tone `row`.

        ..  container:: example

            >>> abjad.PitchClassSegment([-2, -1, 6, 7, -1, 7])
            PitchClassSegment([10, 11, 6, 7, 11, 7])

            >>> segment = abjad.PitchClassSegment([-2, -1, 6, 7, -1, 7])
            >>> abjad.show(segment) # doctest: +SKIP

            ..  doctest:

                >>> lilypond_file = segment.__illustrate__()
                >>> abjad.f(lilypond_file[abjad.Voice])
                \new Voice
                {
                    bf'8
                    b'8
                    fs'8
                    g'8
                    b'8
                    g'8
                    \bar "|." %! SCORE1
                    \override Score.BarLine.transparent = ##f
                }

            >>> segment.permute([10, 0, 2, 6, 8, 7, 5, 3, 1, 9, 4, 11])
            PitchClassSegment([4, 11, 5, 3, 11, 3])

            >>> segment = segment.permute([10, 0, 2, 6, 8, 7, 5, 3, 1, 9, 4, 11])
            >>> abjad.show(segment) # doctest: +SKIP

            ..  docs::

                >>> lilypond_file = segment.__illustrate__()
                >>> abjad.f(lilypond_file[abjad.Voice])
                \new Voice
                {
                    e'8
                    b'8
                    f'8
                    ef'8
                    b'8
                    ef'8
                    \bar "|." %! SCORE1
                    \override Score.BarLine.transparent = ##f
                }

        ..  container:: example expression

            >>> expression = abjad.Expression(name='J')
            >>> expression = expression.pitch_class_segment()
            >>> row = [10, 0, 2, 6, 8, 7, 5, 3, 1, 9, 4, 11]
            >>> expression = expression.permute(row)

            >>> expression([-2, -1, 6, 7, -1, 7])
            PitchClassSegment([4, 11, 5, 3, 11, 3])

            >>> expression.get_string()
            'permute(J, row=[10, 0, 2, 6, 8, 7, 5, 3, 1, 9, 4, 11])'

            >>> segment = expression([-2, -1, 6, 7, -1, 7])
            >>> markup = expression.get_markup()
            >>> abjad.show(segment, figure_name=markup) # doctest: +SKIP

            ..  doctest:

                >>> lilypond_file = segment.__illustrate__(
                ...     figure_name=markup,
                ...     )
                >>> abjad.f(lilypond_file[abjad.Voice])
                \new Voice
                {
                    e'8
                    ^ \markup {
                        \concat
                            {
                                permute(
                                \bold
                                    J
                                ", row=[10, 0, 2, 6, 8, 7, 5, 3, 1, 9, 4, 11])"
                            }
                        }
                    b'8
                    f'8
                    ef'8
                    b'8
                    ef'8
                    \bar "|." %! SCORE1
                    \override Score.BarLine.transparent = ##f
                }

        Returns new segment.
        """
        import abjad
        if self._expression:
            return self._update_expression(inspect.currentframe())
        row = abjad.TwelveToneRow(items=row)
        items = row(self)
        return type(self)(items=items)

    @Signature(is_operator=True, method_name='R')
    def retrograde(self):
        r"""
        Gets retrograde of segment.

        ..  container:: example

            Example segment:

            >>> items = [-2, -1.5, 6, 7, -1.5, 7]
            >>> J = abjad.PitchClassSegment(items=items)
            >>> J
            PitchClassSegment([10, 10.5, 6, 7, 10.5, 7])

            >>> abjad.show(J) # doctest: +SKIP

        ..  container:: example

            Gets retrograde of segment:

            ..  container:: example

                >>> segment = J.retrograde()
                >>> segment
                PitchClassSegment([7, 10.5, 7, 6, 10.5, 10])

                >>> abjad.show(segment) # doctest: +SKIP

                ..  docs::

                    >>> lilypond_file = segment.__illustrate__()
                    >>> abjad.f(lilypond_file[abjad.Voice])
                    \new Voice
                    {
                        g'8
                        bqf'8
                        g'8
                        fs'8
                        bqf'8
                        bf'8
                        \bar "|." %! SCORE1
                        \override Score.BarLine.transparent = ##f
                    }

            ..  container:: example expression

                >>> expression = abjad.Expression(name='J')
                >>> expression = expression.pitch_class_segment()
                >>> expression = expression.retrograde()

                >>> expression([-2, -1.5, 6, 7, -1.5, 7])
                PitchClassSegment([7, 10.5, 7, 6, 10.5, 10])

                >>> expression.get_string()
                'R(J)'

                >>> segment = expression([-2, -1.5, 6, 7, -1.5, 7])
                >>> markup = expression.get_markup()
                >>> abjad.show(segment, figure_name=markup) # doctest: +SKIP

                ..  docs::

                    >>> lilypond_file = segment.__illustrate__(
                    ...     figure_name=markup,
                    ...     )
                    >>> abjad.f(lilypond_file[abjad.Voice])
                    \new Voice
                    {
                        g'8
                        ^ \markup {
                            \concat
                                {
                                    R
                                    \bold
                                        J
                                }
                            }
                        bqf'8
                        g'8
                        fs'8
                        bqf'8
                        bf'8
                        \bar "|." %! SCORE1
                        \override Score.BarLine.transparent = ##f
                    }

        ..  container:: example

            Gets retrograde of retrograde of segment:

            ..  container:: example

                >>> segment = J.retrograde().retrograde()
                >>> segment
                PitchClassSegment([10, 10.5, 6, 7, 10.5, 7])

                >>> abjad.show(segment) # doctest: +SKIP

                ..  docs::

                    >>> lilypond_file = segment.__illustrate__()
                    >>> abjad.f(lilypond_file[abjad.Voice])
                    \new Voice
                    {
                        bf'8
                        bqf'8
                        fs'8
                        g'8
                        bqf'8
                        g'8
                        \bar "|." %! SCORE1
                        \override Score.BarLine.transparent = ##f
                    }

                >>> segment == J
                True

            ..  container:: example expression

                >>> expression = abjad.Expression(name='J')
                >>> expression = expression.pitch_class_segment()
                >>> expression = expression.retrograde()
                >>> expression = expression.retrograde()

                >>> expression([-2, -1.5, 6, 7, -1.5, 7])
                PitchClassSegment([10, 10.5, 6, 7, 10.5, 7])

                >>> expression.get_string()
                'R(R(J))'

                >>> segment = expression([-2, -1.5, 6, 7, -1.5, 7])
                >>> markup = expression.get_markup()
                >>> abjad.show(segment, figure_name=markup) # doctest: +SKIP

                ..  docs::

                    >>> lilypond_file = segment.__illustrate__(
                    ...     figure_name=markup,
                    ...     )
                    >>> abjad.f(lilypond_file[abjad.Voice])
                    \new Voice
                    {
                        bf'8
                        ^ \markup {
                            \concat
                                {
                                    R
                                    \concat
                                        {
                                            R
                                            \bold
                                                J
                                        }
                                }
                            }
                        bqf'8
                        fs'8
                        g'8
                        bqf'8
                        g'8
                        \bar "|." %! SCORE1
                        \override Score.BarLine.transparent = ##f
                    }

                >>> segment == J
                True

        ..  container:: example

            Returns pitch-class segment:

            >>> isinstance(segment, abjad.PitchClassSegment)
            True

        """
        if self._expression:
            return self._update_expression(inspect.currentframe())
        return type(self)(items=reversed(self))

    @Signature(
        is_operator=True,
        method_name_callback='_make_rotate_method_name',
        subscript='n',
        )
    def rotate(self, n=0, stravinsky=False):
        r"""
        Rotates segment by index `n`.

        ..  container:: example

            Example segment:

            >>> items = [-2, -1.5, 6, 7, -1.5, 7]
            >>> J = abjad.PitchClassSegment(items=items)
            >>> J
            PitchClassSegment([10, 10.5, 6, 7, 10.5, 7])

            >>> abjad.show(J) # doctest: +SKIP

        ..  container:: example

            Rotates segment to the right:

            ..  container:: example

                >>> J.rotate(n=1)
                PitchClassSegment([7, 10, 10.5, 6, 7, 10.5])

                >>> segment = J.rotate(n=1)
                >>> abjad.show(segment) # doctest: +SKIP

                ..  docs::

                    >>> lilypond_file = segment.__illustrate__()
                    >>> abjad.f(lilypond_file[abjad.Voice])
                    \new Voice
                    {
                        g'8
                        bf'8
                        bqf'8
                        fs'8
                        g'8
                        bqf'8
                        \bar "|." %! SCORE1
                        \override Score.BarLine.transparent = ##f
                    }

            ..  container:: example expression

                >>> expression = abjad.Expression(name='J')
                >>> expression = expression.pitch_class_segment()
                >>> expression = expression.rotate(n=1)

                >>> expression([-2, -1.5, 6, 7, -1.5, 7])
                PitchClassSegment([7, 10, 10.5, 6, 7, 10.5])

                >>> expression.get_string()
                'r1(J)'

                >>> segment = expression([-2, -1.5, 6, 7, -1.5, 7])
                >>> markup = expression.get_markup()
                >>> abjad.show(segment, figure_name=markup) # doctest: +SKIP

                ..  docs::

                    >>> lilypond_file = segment.__illustrate__(
                    ...     figure_name=markup,
                    ...     )
                    >>> abjad.f(lilypond_file[abjad.Voice])
                    \new Voice
                    {
                        g'8
                        ^ \markup {
                            \concat
                                {
                                    r
                                    \sub
                                        1
                                    \bold
                                        J
                                }
                            }
                        bf'8
                        bqf'8
                        fs'8
                        g'8
                        bqf'8
                        \bar "|." %! SCORE1
                        \override Score.BarLine.transparent = ##f
                    }

        ..  container:: example

            Rotates segment to the left:

            ..  container:: example

                >>> J.rotate(n=-1)
                PitchClassSegment([10.5, 6, 7, 10.5, 7, 10])

                >>> segment = J.rotate(n=-1)
                >>> abjad.show(segment) # doctest: +SKIP

                ..  docs::

                    >>> lilypond_file = segment.__illustrate__()
                    >>> abjad.f(lilypond_file[abjad.Voice])
                    \new Voice
                    {
                        bqf'8
                        fs'8
                        g'8
                        bqf'8
                        g'8
                        bf'8
                        \bar "|." %! SCORE1
                        \override Score.BarLine.transparent = ##f
                    }

            ..  container:: example expression

                >>> expression = abjad.Expression(name='J')
                >>> expression = expression.pitch_class_segment()
                >>> expression = expression.rotate(n=-1)

                >>> expression([-2, -1.5, 6, 7, -1.5, 7])
                PitchClassSegment([10.5, 6, 7, 10.5, 7, 10])

                >>> expression.get_string()
                'r-1(J)'

                >>> segment = expression([-2, -1.5, 6, 7, -1.5, 7])
                >>> markup = expression.get_markup()
                >>> abjad.show(segment, figure_name=markup) # doctest: +SKIP

                ..  docs::

                    >>> lilypond_file = segment.__illustrate__(
                    ...     figure_name=markup,
                    ...     )
                    >>> abjad.f(lilypond_file[abjad.Voice])
                    \new Voice
                    {
                        bqf'8
                        ^ \markup {
                            \concat
                                {
                                    r
                                    \sub
                                        -1
                                    \bold
                                        J
                                }
                            }
                        fs'8
                        g'8
                        bqf'8
                        g'8
                        bf'8
                        \bar "|." %! SCORE1
                        \override Score.BarLine.transparent = ##f
                    }

        ..  container:: example

            Rotates segment by zero:

            ..  container:: example

                >>> J.rotate(n=0)
                PitchClassSegment([10, 10.5, 6, 7, 10.5, 7])

                >>> segment = J.rotate(n=0)
                >>> abjad.show(segment) # doctest: +SKIP

                ..  docs::

                    >>> lilypond_file = segment.__illustrate__()
                    >>> abjad.f(lilypond_file[abjad.Voice])
                    \new Voice
                    {
                        bf'8
                        bqf'8
                        fs'8
                        g'8
                        bqf'8
                        g'8
                        \bar "|." %! SCORE1
                        \override Score.BarLine.transparent = ##f
                    }

                >>> segment == J
                True

            ..  container:: example expression

                >>> expression = abjad.Expression(name='J')
                >>> expression = expression.pitch_class_segment()
                >>> expression = expression.rotate(n=0)

                >>> expression([-2, -1.5, 6, 7, -1.5, 7])
                PitchClassSegment([10, 10.5, 6, 7, 10.5, 7])

                >>> expression.get_string()
                'r0(J)'

                >>> segment = expression([-2, -1.5, 6, 7, -1.5, 7])
                >>> markup = expression.get_markup()
                >>> abjad.show(segment, figure_name=markup) # doctest: +SKIP

                ..  docs::

                    >>> lilypond_file = segment.__illustrate__(
                    ...     figure_name=markup,
                    ...     )
                    >>> abjad.f(lilypond_file[abjad.Voice])
                    \new Voice
                    {
                        bf'8
                        ^ \markup {
                            \concat
                                {
                                    r
                                    \sub
                                        0
                                    \bold
                                        J
                                }
                            }
                        bqf'8
                        fs'8
                        g'8
                        bqf'8
                        g'8
                        \bar "|." %! SCORE1
                        \override Score.BarLine.transparent = ##f
                    }

                >>> segment == J
                True

        ..  container:: example

            Stravinsky-style rotation back-transposes segment to
            begin at zero:

            ..  container:: example

                >>> J.rotate(n=1, stravinsky=True)
                PitchClassSegment([0, 3, 3.5, 11, 0, 3.5])

                >>> segment = J.rotate(n=1, stravinsky=True)
                >>> abjad.show(segment) # doctest: +SKIP

                ..  docs::

                    >>> lilypond_file = segment.__illustrate__()
                    >>> abjad.f(lilypond_file[abjad.Voice])
                    \new Voice
                    {
                        c'8
                        ef'8
                        eqf'8
                        b'8
                        c'8
                        eqf'8
                        \bar "|." %! SCORE1
                        \override Score.BarLine.transparent = ##f
                    }

            ..  container:: example expression

                >>> expression = abjad.Expression(name='J')
                >>> expression = expression.pitch_class_segment()
                >>> expression = expression.rotate(n=1, stravinsky=True)

                >>> expression([-2, -1.5, 6, 7, -1.5, 7])
                PitchClassSegment([0, 3, 3.5, 11, 0, 3.5])

                >>> expression.get_string()
                'rs1(J)'

                >>> segment = expression([-2, -1.5, 6, 7, -1.5, 7])
                >>> markup = expression.get_markup()
                >>> abjad.show(segment, figure_name=markup) # doctest: +SKIP

                ..  docs::

                    >>> lilypond_file = segment.__illustrate__(
                    ...     figure_name=markup,
                    ...     )
                    >>> abjad.f(lilypond_file[abjad.Voice])
                    \new Voice
                    {
                        c'8
                        ^ \markup {
                            \concat
                                {
                                    rs
                                    \sub
                                        1
                                    \bold
                                        J
                                }
                            }
                        ef'8
                        eqf'8
                        b'8
                        c'8
                        eqf'8
                        \bar "|." %! SCORE1
                        \override Score.BarLine.transparent = ##f
                    }

        ..  container:: example

            Returns pitch-class segment:

            >>> isinstance(segment, abjad.PitchClassSegment)
            True

        """
        import abjad
        if self._expression:
            return self._update_expression(inspect.currentframe())
        items = abjad.sequence(self._collection).rotate(n=n)
        if stravinsky:
            n = 0 - float(items[0].number)
            segment = new(self, items=items)
            segment = segment.transpose(n=n)
            items = segment.items[:]
        return type(self)(items=items)

    def to_pitch_classes(self):
        r"""
        Changes to pitch-classes.

        ..  container:: example

            To numbered pitch-class segment:

            >>> items = [-2, -1.5, 6, 7, -1.5, 7]
            >>> segment = abjad.PitchClassSegment(items=items)
            >>> segment
            PitchClassSegment([10, 10.5, 6, 7, 10.5, 7])

            >>> abjad.show(segment) # doctest: +SKIP

            ..  docs::

                >>> lilypond_file = segment.__illustrate__()
                >>> abjad.f(lilypond_file[abjad.Voice])
                \new Voice
                {
                    bf'8
                    bqf'8
                    fs'8
                    g'8
                    bqf'8
                    g'8
                    \bar "|." %! SCORE1
                    \override Score.BarLine.transparent = ##f
                }

            >>> segment = segment.to_pitch_classes()
            >>> segment
            PitchClassSegment([10, 10.5, 6, 7, 10.5, 7])

            >>> abjad.show(segment) # doctest: +SKIP

            ..  docs::

                >>> lilypond_file = segment.__illustrate__()
                >>> abjad.f(lilypond_file[abjad.Voice])
                \new Voice
                {
                    bf'8
                    bqf'8
                    fs'8
                    g'8
                    bqf'8
                    g'8
                    \bar "|." %! SCORE1
                    \override Score.BarLine.transparent = ##f
                }

        ..  container:: example

            To named pitch-class segment:

            >>> items = [-2, -1.5, 6, 7, -1.5, 7]
            >>> segment = abjad.PitchClassSegment(
            ...     items=items,
            ...     item_class=abjad.NamedPitchClass,
            ...     )
            >>> segment
            PitchClassSegment("bf bqf fs g bqf g")

            >>> abjad.show(segment) # doctest: +SKIP

            ..  docs::

                >>> lilypond_file = segment.__illustrate__()
                >>> abjad.f(lilypond_file[abjad.Voice])
                \new Voice
                {
                    bf'8
                    bqf'8
                    fs'8
                    g'8
                    bqf'8
                    g'8
                    \bar "|." %! SCORE1
                    \override Score.BarLine.transparent = ##f
                }

            >>> segment = segment.to_pitch_classes()
            >>> segment
            PitchClassSegment("bf bqf fs g bqf g")

            >>> abjad.show(segment) # doctest: +SKIP

            ..  docs::

                >>> lilypond_file = segment.__illustrate__()
                >>> abjad.f(lilypond_file[abjad.Voice])
                \new Voice
                {
                    bf'8
                    bqf'8
                    fs'8
                    g'8
                    bqf'8
                    g'8
                    \bar "|." %! SCORE1
                    \override Score.BarLine.transparent = ##f
                }

        Returns new segment.
        """
        return new(self)

    def to_pitches(self):
        r"""
        Changes to pitches.

        ..  container:: example

            To numbered pitch segment:

            >>> items = [-2, -1.5, 6, 7, -1.5, 7]
            >>> segment = abjad.PitchClassSegment(items=items)
            >>> segment
            PitchClassSegment([10, 10.5, 6, 7, 10.5, 7])

            >>> abjad.show(segment) # doctest: +SKIP

            ..  docs::

                >>> lilypond_file = segment.__illustrate__()
                >>> abjad.f(lilypond_file[abjad.Voice])
                \new Voice
                {
                    bf'8
                    bqf'8
                    fs'8
                    g'8
                    bqf'8
                    g'8
                    \bar "|." %! SCORE1
                    \override Score.BarLine.transparent = ##f
                }

            >>> segment = segment.to_pitches()
            >>> segment
            PitchSegment([10, 10.5, 6, 7, 10.5, 7])

            >>> abjad.show(segment) # doctest: +SKIP

            ..  docs::

                >>> lilypond_file = segment.__illustrate__()
                >>> abjad.f(lilypond_file[abjad.StaffGroup])
                \new PianoStaff
                <<
                    \context Staff = "Treble Staff"
                    {
                        \clef "treble"
                        bf'1 * 1/8
                        bqf'1 * 1/8
                        fs'1 * 1/8
                        g'1 * 1/8
                        bqf'1 * 1/8
                        g'1 * 1/8
                    }
                    \context Staff = "Bass Staff"
                    {
                        \clef "bass"
                        r1 * 1/8
                        r1 * 1/8
                        r1 * 1/8
                        r1 * 1/8
                        r1 * 1/8
                        r1 * 1/8
                    }
                >>

        ..  container:: example

            To named pitch segment:

            >>> items = [-2, -1.5, 6, 7, -1.5, 7]
            >>> segment = abjad.PitchClassSegment(
            ...     items=items,
            ...     item_class=abjad.NamedPitchClass,
            ...     )
            >>> segment
            PitchClassSegment("bf bqf fs g bqf g")

            >>> abjad.show(segment) # doctest: +SKIP

            ..  docs::

                >>> lilypond_file = segment.__illustrate__()
                >>> abjad.f(lilypond_file[abjad.Voice])
                \new Voice
                {
                    bf'8
                    bqf'8
                    fs'8
                    g'8
                    bqf'8
                    g'8
                    \bar "|." %! SCORE1
                    \override Score.BarLine.transparent = ##f
                }

            >>> segment = segment.to_pitches()
            >>> segment
            PitchSegment("bf' bqf' fs' g' bqf' g'")

            >>> abjad.show(segment) # doctest: +SKIP

            ..  docs::

                >>> lilypond_file = segment.__illustrate__()
                >>> abjad.f(lilypond_file[abjad.StaffGroup])
                \new PianoStaff
                <<
                    \context Staff = "Treble Staff"
                    {
                        \clef "treble"
                        bf'1 * 1/8
                        bqf'1 * 1/8
                        fs'1 * 1/8
                        g'1 * 1/8
                        bqf'1 * 1/8
                        g'1 * 1/8
                    }
                    \context Staff = "Bass Staff"
                    {
                        \clef "bass"
                        r1 * 1/8
                        r1 * 1/8
                        r1 * 1/8
                        r1 * 1/8
                        r1 * 1/8
                        r1 * 1/8
                    }
                >>

        Returns new segment.
        """
        import abjad
        class_ = abjad.Pitch
        item_class = class_._to_pitch_item_class(self.item_class)
        return abjad.PitchSegment(items=self.items, item_class=item_class)

    @Signature(
        is_operator=True,
        method_name='T',
        subscript='n',
        )
    def transpose(self, n=0):
        r"""
        Transposes segment by index `n`.

        ..  container:: example

            Example segment:

            >>> items = [-2, -1.5, 6, 7, -1.5, 7]
            >>> J = abjad.PitchClassSegment(items=items)
            >>> J
            PitchClassSegment([10, 10.5, 6, 7, 10.5, 7])

            >>> abjad.show(J) # doctest: +SKIP

        ..  container:: example

            Transposes segment by positive index:

            ..  container:: example

                >>> J.transpose(n=13)
                PitchClassSegment([11, 11.5, 7, 8, 11.5, 8])

                >>> segment = J.transpose(n=13)
                >>> abjad.show(segment) # doctest: +SKIP

                ..  docs::

                    >>> lilypond_file = segment.__illustrate__()
                    >>> abjad.f(lilypond_file[abjad.Voice])
                    \new Voice
                    {
                        b'8
                        bqs'8
                        g'8
                        af'8
                        bqs'8
                        af'8
                        \bar "|." %! SCORE1
                        \override Score.BarLine.transparent = ##f
                    }

            ..  container:: example expression

                >>> expression = abjad.Expression(name='J')
                >>> expression = expression.pitch_class_segment()
                >>> expression = expression.transpose(n=13)

                >>> expression([-2, -1.5, 6, 7, -1.5, 7])
                PitchClassSegment([11, 11.5, 7, 8, 11.5, 8])

                >>> expression.get_string()
                'T13(J)'

                >>> segment = expression([-2, -1.5, 6, 7, -1.5, 7])
                >>> markup = expression.get_markup()
                >>> abjad.show(segment, figure_name=markup) # doctest: +SKIP

                ..  docs::

                    >>> lilypond_file = segment.__illustrate__(
                    ...     figure_name=markup,
                    ...     )
                    >>> abjad.f(lilypond_file[abjad.Voice])
                    \new Voice
                    {
                        b'8
                        ^ \markup {
                            \concat
                                {
                                    T
                                    \sub
                                        13
                                    \bold
                                        J
                                }
                            }
                        bqs'8
                        g'8
                        af'8
                        bqs'8
                        af'8
                        \bar "|." %! SCORE1
                        \override Score.BarLine.transparent = ##f
                    }

        ..  container:: example

            Transposes segment by negative index:

            ..  container:: example

                >>> J.transpose(n=-13)
                PitchClassSegment([9, 9.5, 5, 6, 9.5, 6])

                >>> segment = J.transpose(n=-13)
                >>> abjad.show(segment) # doctest: +SKIP

                ..  docs::

                    >>> lilypond_file = segment.__illustrate__()
                    >>> abjad.f(lilypond_file[abjad.Voice])
                    \new Voice
                    {
                        a'8
                        aqs'8
                        f'8
                        fs'8
                        aqs'8
                        fs'8
                        \bar "|." %! SCORE1
                        \override Score.BarLine.transparent = ##f
                    }

            ..  container:: example expression

                >>> expression = abjad.Expression(name='J')
                >>> expression = expression.pitch_class_segment()
                >>> expression = expression.transpose(n=-13)

                >>> expression([-2, -1.5, 6, 7, -1.5, 7])
                PitchClassSegment([9, 9.5, 5, 6, 9.5, 6])

                >>> expression.get_string()
                'T-13(J)'

                >>> segment = expression([-2, -1.5, 6, 7, -1.5, 7])
                >>> markup = expression.get_markup()
                >>> abjad.show(segment, figure_name=markup) # doctest: +SKIP

                ..  docs::

                    >>> lilypond_file = segment.__illustrate__(
                    ...     figure_name=markup,
                    ...     )
                    >>> abjad.f(lilypond_file[abjad.Voice])
                    \new Voice
                    {
                        a'8
                        ^ \markup {
                            \concat
                                {
                                    T
                                    \sub
                                        -13
                                    \bold
                                        J
                                }
                            }
                        aqs'8
                        f'8
                        fs'8
                        aqs'8
                        fs'8
                        \bar "|." %! SCORE1
                        \override Score.BarLine.transparent = ##f
                    }

        ..  container:: example

            Transposes segment by zero index:

            ..  container:: example

                >>> J.transpose(n=0)
                PitchClassSegment([10, 10.5, 6, 7, 10.5, 7])

                >>> segment = J.transpose(n=0)
                >>> abjad.show(segment) # doctest: +SKIP

                ..  docs::

                    >>> lilypond_file = segment.__illustrate__()
                    >>> abjad.f(lilypond_file[abjad.Voice])
                    \new Voice
                    {
                        bf'8
                        bqf'8
                        fs'8
                        g'8
                        bqf'8
                        g'8
                        \bar "|." %! SCORE1
                        \override Score.BarLine.transparent = ##f
                    }

                >>> segment == J
                True

            ..  container:: example expression

                >>> expression = abjad.Expression(name='J')
                >>> expression = expression.pitch_class_segment()
                >>> expression = expression.transpose(n=0)

                >>> expression([-2, -1.5, 6, 7, -1.5, 7])
                PitchClassSegment([10, 10.5, 6, 7, 10.5, 7])

                >>> expression.get_string()
                'T0(J)'

                >>> segment = expression([-2, -1.5, 6, 7, -1.5, 7])
                >>> markup = expression.get_markup()
                >>> abjad.show(segment, figure_name=markup) # doctest: +SKIP

                ..  docs::

                    >>> lilypond_file = segment.__illustrate__(
                    ...     figure_name=markup,
                    ...     )
                    >>> abjad.f(lilypond_file[abjad.Voice])
                    \new Voice
                    {
                        bf'8
                        ^ \markup {
                            \concat
                                {
                                    T
                                    \sub
                                        0
                                    \bold
                                        J
                                }
                            }
                        bqf'8
                        fs'8
                        g'8
                        bqf'8
                        g'8
                        \bar "|." %! SCORE1
                        \override Score.BarLine.transparent = ##f
                    }

                >>> segment == J
                True

        ..  container:: example

            Returns pitch-class segment:

            >>> isinstance(segment, abjad.PitchClassSegment)
            True

        """
        if self._expression:
            return self._update_expression(inspect.currentframe())
        items = [_.transpose(n=n) for _ in self]
        return type(self)(items=items)

    def voice_horizontally(self, initial_octave=4):
        r"""
        Voices segment with each pitch as close to the previous pitch as
        possible.

        ..  todo:: Should be implemented somewhere else.

        ..  container:: example

            Voices horizontally:

            >>> items = "c b d e f g e b a c"
            >>> segment = abjad.PitchClassSegment(items=items)
            >>> abjad.show(segment) # doctest: +SKIP

            >>> voiced_segment = segment.voice_horizontally()
            >>> abjad.show(voiced_segment) # doctest: +SKIP

            ..  docs::

                >>> lilypond_file = voiced_segment.__illustrate__()
                >>> abjad.f(lilypond_file[abjad.Score])
                \new Score
                \with
                {
                    \override BarLine.stencil = ##f
                    \override BarNumber.transparent = ##t
                    \override Rest.transparent = ##t
                    \override SpanBar.stencil = ##f
                    \override TimeSignature.stencil = ##f
                }
                <<
                    \new PianoStaff
                    <<
                        \context Staff = "Treble Staff"
                        {
                            \clef "treble"
                            c'1 * 1/8
                            b1 * 1/8
                            d'1 * 1/8
                            e'1 * 1/8
                            f'1 * 1/8
                            g'1 * 1/8
                            e'1 * 1/8
                            b1 * 1/8
                            r1 * 1/8
                            c'1 * 1/8
                        }
                        \context Staff = "Bass Staff"
                        {
                            \clef "bass"
                            r1 * 1/8
                            r1 * 1/8
                            r1 * 1/8
                            r1 * 1/8
                            r1 * 1/8
                            r1 * 1/8
                            r1 * 1/8
                            r1 * 1/8
                            a1 * 1/8
                            r1 * 1/8
                        }
                    >>
                >>

        ..  container:: example

            Returns pitch segment:

            >>> voiced_segment
            PitchSegment("c' b d' e' f' g' e' b a c'")

        """
        import abjad
        initial_octave = abjad.Octave(initial_octave)
        pitches = []
        if self:
            pitch_class = abjad.NamedPitchClass(self[0])
            pitch = abjad.NamedPitch((pitch_class.name, initial_octave))
            pitches.append(pitch)
            for pitch_class in self[1:]:
                pitch_class = abjad.NamedPitchClass(pitch_class)
                pitch = abjad.NamedPitch((pitch_class.name, initial_octave))
                semitones = abs((pitch - pitches[-1]).semitones)
                while 6 < semitones:
                    if pitch < pitches[-1]:
                        pitch += 12
                    else:
                        pitch -= 12
                    semitones = abs((pitch - pitches[-1]).semitones)
                pitches.append(pitch)
        if self.item_class is abjad.NamedPitchClass:
            item_class = abjad.NamedPitch
        else:
            item_class = abjad.NumberedPitch
        return abjad.PitchSegment(
            items=pitches,
            item_class=item_class,
            )

    def voice_vertically(self, initial_octave=4):
        r"""
        Voices segment with each pitch higher than the previous.

        ..  todo:: Should be implemented somewhere else.

        ..  container:: example

            Voices vertically:

            >>> string = "c' ef' g' bf' d'' f'' af''"
            >>> segment = abjad.PitchClassSegment(string)
            >>> abjad.show(segment) # doctest: +SKIP

            >>> voiced_segment = segment.voice_vertically()
            >>> abjad.show(voiced_segment) # doctest: +SKIP

            ..  docs::

                >>> lilypond_file = voiced_segment.__illustrate__()
                >>> abjad.f(lilypond_file[abjad.Score])
                \new Score
                \with
                {
                    \override BarLine.stencil = ##f
                    \override BarNumber.transparent = ##t
                    \override Rest.transparent = ##t
                    \override SpanBar.stencil = ##f
                    \override TimeSignature.stencil = ##f
                }
                <<
                    \new PianoStaff
                    <<
                        \context Staff = "Treble Staff"
                        {
                            \clef "treble"
                            c'1 * 1/8
                            ef'1 * 1/8
                            g'1 * 1/8
                            bf'1 * 1/8
                            d''1 * 1/8
                            f''1 * 1/8
                            af''1 * 1/8
                        }
                        \context Staff = "Bass Staff"
                        {
                            \clef "bass"
                            r1 * 1/8
                            r1 * 1/8
                            r1 * 1/8
                            r1 * 1/8
                            r1 * 1/8
                            r1 * 1/8
                            r1 * 1/8
                        }
                    >>
                >>

        ..  container:: example

            Returns pitch segment:

            >>> voiced_segment
            PitchSegment("c' ef' g' bf' d'' f'' af''")

        """
        import abjad
        initial_octave = abjad.Octave(initial_octave)
        pitches = []
        if self:
            pitch_class = abjad.NamedPitchClass(self[0])
            pitch = abjad.NamedPitch((pitch_class.name, initial_octave))
            pitches.append(pitch)
            for pitch_class in self[1:]:
                pitch_class = abjad.NamedPitchClass(pitch_class)
                pitch = abjad.NamedPitch((pitch_class.name, initial_octave))
                while pitch < pitches[-1]:
                    pitch += 12
                pitches.append(pitch)
        if self.item_class is abjad.NamedPitchClass:
            item_class = abjad.NamedPitch
        else:
            item_class = abjad.NumberedPitch
        return abjad.PitchSegment(
            items=pitches,
            item_class=item_class,
            )



import typing
from abjad.system.AbjadObject import AbjadObject
from abjad.utilities.String import String
from abjad.system.FormatSpecification import FormatSpecification


class Part(AbjadObject):
    """
    Part.

    ..  container:: example

        >>> part = abjad.Part(
        ...     member=18,
        ...     section='FirstViolin',
        ...     section_abbreviation='VN-1',
        ...     )
        
        >>> abjad.f(part)
        abjad.Part(
            instrument='FirstViolin',
            member=18,
            section='FirstViolin',
            section_abbreviation='VN-1',
            )

    """

    ### CLASS VARIABLES ###

    __slots__ = (
        '_section_abbreviation',
        '_instrument',
        '_member',
        '_name',
        '_number',
        '_section',
        '_zfill',
        )

    _publish_storage_format = True

    ### INITIALIZER ###

    def __init__(
        self,
        instrument: str = None,
        member: int = None,
        number: int = None,
        section: str = None,
        section_abbreviation: str = None,
        zfill: int = None,
        ) -> None:
        instrument = instrument or section
        if instrument is not None:
            if not isinstance(instrument, str):
                message = 'instrument must be string'
                message += f' (not {instrument!r}).'
                raise Exception(message)
        self._instrument = instrument
        if member is not None:
            if not isinstance(member, int):
                message = 'member must be integer'
                message += f' (not {member!r}).'
                raise Exception(message)
        self._member = member
        if number is not None:
            assert isinstance(number, int), repr(number)
            assert 1 <= number, repr(number)
        self._number = number
        if section is not None:
            if not isinstance(section, str):
                raise Exception(f'section must be string (not {section!r}).')
        self._section = section
        if section_abbreviation is not None:
            if not isinstance(section_abbreviation, str):
                message = 'section_abbreviation must be string'
                message += f' (not {section_abbreviation!r}).'
                raise Exception(message)
        self._section_abbreviation = section_abbreviation
        if zfill is not None:
            assert isinstance(zfill, int), repr(zfill)
            assert 1 <= zfill, repr(zfill)
        self._zfill = zfill
        if member is not None:
            member_ = str(member)
            if self.zfill is not None:
                member_ = member_.zfill(self.zfill)
            name: typing.Optional[str] = f'{section}{member_}'
        else:
            name = section
        self._name = name

    ### SPECIAL METHODS ###

    def __eq__(self, argument) -> bool:
        """
        Is true when ``argument`` is a part with the same section and
        member as this part.

        ..  container:: example

            >>> part_1 = abjad.Part(
            ...     member=18,
            ...     section='FirstViolin',
            ...     section_abbreviation='VN-1',
            ...     )
            >>> part_2 = abjad.Part(
            ...     member=18,
            ...     section='FirstViolin',
            ...     section_abbreviation='VN-1',
            ...     )
            >>> part_3 = abjad.Part(
            ...     member=18,
            ...     section='SecondViolin',
            ...     section_abbreviation='VN-2',
            ...     )

            >>> part_1 == part_1
            True
            >>> part_1 == part_2
            True
            >>> part_1 == part_3
            False

            >>> part_2 == part_1
            True
            >>> part_2 == part_2
            True
            >>> part_2 == part_3
            False

            >>> part_3 == part_1
            False
            >>> part_3 == part_2
            False
            >>> part_3 == part_3
            True

        """
        if isinstance(argument, type(self)):
            if argument.section == self.section:
                return argument.member == self.member
        return False

    def __hash__(self):
        """
        Hashes part.
        """
        return super().__hash__()

    ### PUBLIC PROPERTIES ###

    @property
    def identifier(self) -> str:
        """
        Gets identifier.

        ..  container:: example

            >>> part = abjad.Part(
            ...     instrument='Violin',
            ...     member=18,
            ...     section='FirstViolin',
            ...     section_abbreviation='VN-1',
            ...     )

            >>> part.identifier
            'VN-1-18'

        """
        assert isinstance(self.section_abbreviation, str)
        if self.member is None:
            return self.section_abbreviation
        else:
            assert isinstance(self.member, int)
            return f'{self.section_abbreviation}-{self.member}'

    @property
    def instrument(self) -> typing.Optional[str]:
        """
        Gets instrument.

        ..  container:: example

            >>> part = abjad.Part(
            ...     instrument='Violin',
            ...     member=18,
            ...     section='FirstViolin',
            ...     section_abbreviation='VN-1',
            ...     )

            >>> part.instrument
            'Violin'

        """
        return self._instrument

    @property
    def member(self) -> typing.Optional[int]:
        """
        Gets member.

        ..  container:: example

            >>> part = abjad.Part(
            ...     instrument='Violin',
            ...     member=18,
            ...     section='FirstViolin',
            ...     section_abbreviation='VN-1',
            ...     )

            >>> part.member
            18

        """
        return self._member

    @property
    def name(self) -> typing.Optional[str]:
        """
        Gets name.

        ..  container:: example

            >>> part = abjad.Part(
            ...     instrument='Violin',
            ...     member=1,
            ...     section='FirstViolin',
            ...     section_abbreviation='VN-1',
            ...     )

            >>> part.name
            'FirstViolin1'

            >>> part = abjad.Part(
            ...     instrument='Violin',
            ...     member=1,
            ...     zfill=2,
            ...     section='FirstViolin',
            ...     section_abbreviation='VN-1',
            ...     )

            >>> part.name
            'FirstViolin01'

        """
        return self._name

    @property
    def number(self) -> typing.Optional[int]:
        """
        Gets number.

        ..  container:: example

            >>> part = abjad.Part(
            ...     instrument='Violin',
            ...     member=18,
            ...     number=107,
            ...     section='FirstViolin',
            ...     section_abbreviation='VN-1',
            ...     )
            
            >>> part.number
            107

        """
        return self._number

    @property
    def section(self) -> typing.Optional[str]:
        """
        Gets section.

        ..  container:: example

            >>> part = abjad.Part(
            ...     instrument='Violin',
            ...     member=18,
            ...     section='FirstViolin',
            ...     section_abbreviation='VN-1',
            ...     )

            >>> part.section
            'FirstViolin'

        """
        return self._section

    @property
    def section_abbreviation(self) -> typing.Optional[str]:
        """
        Gets section_abbreviation.

        ..  container:: example

            >>> part = abjad.Part(
            ...     instrument='Violin',
            ...     member=18,
            ...     section='FirstViolin',
            ...     section_abbreviation='VN-1',
            ...     )

            >>> part.section_abbreviation
            'VN-1'

        """
        return self._section_abbreviation

    @property
    def zfill(self) -> typing.Optional[int]:
        """
        Gets zfill.

        ..  container:: example

            >>> part = abjad.Part(
            ...     instrument='Violin',
            ...     member=9,
            ...     number=99,
            ...     section='FirstViolin',
            ...     section_abbreviation='VN-1',
            ...     zfill=2,
            ...     )
            
            >>> part.zfill
            2

            >>> str(part.member).zfill(part.zfill)
            '09'

        """
        return self._zfill
